#!/bin/env python
# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# The script keeps the consistent configuration of the network interfaces
# provided by the cloud-init over the reboots of the vm by the following steps:
#
# On first boot (dump file does not exist) or if the '--init' param is used:
#  * Dumps the configuration provided by the cloud-init
#  * The configuration placed into a json file within the instance
#  * If a "tag" provided within the port meta-data, use it to set the selected
#    interface as the default gateway with the lowest metric.
#  * The route metric value stored within the dump for the required interface.
#  * Set the SSH configuration to "UseDNS no" to prevent login delay.
# On the reboot
#  * After the reboot, the script restores the network configuration from the
#    dump stored previously including the route metric.
#
# For the proper execution, place the script within the location that will
# execute the script over each reboot:
# "/var/lib/cloud/scripts/per-boot/"

import argparse
import json
import logging
import os
import re
import subprocess
import sys

logging.basicConfig(filename='/var/log/messages', filemode='a',
                    format='%(asctime)s %(name)s %(levelname)s: %(message)s',
                    datefmt="%h %d %H:%M:%S", level=logging.INFO)
handler = logging.StreamHandler(sys.stdout)
logger = logging.getLogger("CustomNetConfig")
logger.addHandler(handler)


parser = argparse.ArgumentParser(
    description='Multi-interface network config script')
parser.add_argument('-i', '--init', help='Force creation of the network '
                                         'config dump')
parser.add_argument('-t', '--tag', help='The tag assignment is used for '
                                        'the default gateway '
                                        'configuration. The tag assigned '
                                        'to port, gets highest metric '
                                        'priority',
                    default="external")
parser.add_argument('--nics_data_path', help='Location of the json data '
                                             'file to store nics '
                                             'configuration',
                    default='/var/lib/cloud/data/custom_net_config_dump.json')
args = parser.parse_args()


def cloud_init_installed():
    """Verify cloud-init installed"""
    if os.path.exists('/etc/cloud/cloud.cfg'):
        return True
    return False


def configure_ssh_dns():
    """Set SSH to not query for DNS resolve. Solve delay during login."""
    with open('/etc/ssh/sshd_config', 'r+') as f:
        ssh_config = f.read()
        if re.search(r'UseDNS no', ssh_config):
            logger.info('The SSH "UseDNS no" configuration found.')
        else:
            f.write('\nUseDNS no')
            f.flush()
            logger.info('Write "UseDNS no" to SSH config')
            execute_shell_command('systemctl restart sshd')


def check_existing_interfaces():
    """Check and locate existing network interfaces on instance"""
    ifaces = []
    nics = os.listdir('/sys/class/net/')
    nics.remove('lo')
    for nic in nics:
        nic_path = '/sys/class/net/{}/address'.format(nic)
        with open(nic_path, 'r') as f:
            mac = f.read().rstrip()
        ifaces.append({nic: mac})
    return ifaces


def get_ifcfg_files():
    """Get ifaces with configuration file"""
    files = os.listdir('/etc/sysconfig/network-scripts/')
    ifaces = [file.replace('ifcfg-', '') for file in files if 'ifcfg' in file]
    ifaces.remove('lo')
    return ifaces


def verify_interfaces_config(ifaces, tag):
    """Verify interfaces configuration

       In older versions of OSP, the metadata could be provided partially.
       As a result, just one single interface of multiple interfaces is
       configured by the cloud-init within the ifcfg files.
       In such, verification of the interfaces should be done.
       In case a single interface configuration is detected in multiple
       interface instance, only the external interface will be configured.
       Using the mac address of the external interface, the proper ifcfg-eth
       file will be configured.
    """
    files = get_ifcfg_files()
    if set(files) == set([nic.keys()[0] for nic in ifaces]):
        return ifaces
    logger.info('Applying osp10 workaroud. Expected '
                '{} network files, found {}'.format(len(ifaces),
                                                    ' '.join(files)))
    for nic in ifaces:
        for nic_name, nic_mac in iter(nic.items()):
            if tag and nic_mac == tag['mac'] and tag['tag'] == args.tag:
                file_name_template = '/etc/sysconfig/network-scripts/ifcfg-{}'
                source_file = file_name_template.format(files[0])
                destination_file = file_name_template.format(nic_name)
                os.rename(source_file, destination_file)
                iface_dump = []
                try:
                    with open(destination_file, 'r') as iface_file:
                        iface_dump = iface_file.readlines()
                    with open(destination_file, 'w') as iface_file:
                        for line in iface_dump:
                            if line.strip().startswith('DEVICE='):
                                line = 'DEVICE={}'.format(nic_name)
                            elif line.strip().startswith('HWADDR'):
                                line = 'HWADDR={}'.format(nic_mac)
                            iface_file.write('{}\n'.format(line))
                except IOError:
                    logger.info('Unable to read/write '
                                '{} file.'.format(destination_file))
                logger.info('Applied osp10 workaround')
                return [nic]
    msg = 'Not able to apply osp10 workaround'
    logger.info(msg)
    raise ValueError(msg)


def dump_interfaces_config(nics=None, tag=None):
    """Dumps ifcfg files data and route metric"""
    if not nics:
        msg = 'The "nics" dict should hold ifaces macs. Currently empty.'
        logger.info(msg)
        raise ValueError(msg)

    iface_dump = []
    iface_content = {}
    data_path = args.nics_data_path
    for nic in nics:
        for nic_name, nic_mac in iter(nic.items()):
            ifcfg_path = '/etc/sysconfig/network-scripts/' \
                         'ifcfg-{}'.format(nic_name)
            if os.path.exists(ifcfg_path):
                logger.info('The {} interface found'.format(nic_name))
                try:
                    with open(ifcfg_path, 'r') as iface_file:
                        iface_dump = iface_file.readlines()
                        logger.info('Read {} file content'.format(ifcfg_path))
                except IOError:
                    logger.info('Unable to read {} file.'.format(ifcfg_path))

                iface_content[nic_mac] = []
                for opt in iface_dump:
                    iface_content[nic_mac].append(opt)

                if tag and nic_mac == tag['mac'] and tag['tag'] == args.tag:
                    logger.info('Dump lower gateway metric for {} '
                                'interface'.format(nic_name))
                    iface_content[nic_mac].append('IPV4_ROUTE_METRIC=10\n')
            else:
                logger.info('The {} interface is missing'.format(nic_name))
    try:
        with open(data_path, 'w') as jsonfile:
            json.dump(iface_content, jsonfile)
            logger.info('Write ifaces data to {}'.format(data_path))
    except IOError:
        logger.info('Unable to write data to {} file'.format(data_path))
    logger.info('Network data has been successfully dumped')


def recreate_interfaces_config(dump_config_file, nics=None):
    """Recreate interfaces config based on the dumped data"""
    if not dump_config_file or not nics:
        msg = 'Missing one of the following: nics list or interfaces dump.'
        logger.info(msg)
        raise ValueError(msg)

    logger.info('Read interfaces dump file')
    with open(dump_config_file) as json_file:
        dump_config = json.load(json_file)

    for nic in nics:
        for nic_name, nic_mac in iter(nic.items()):
            ifcfg_path = '/etc/sysconfig/network-scripts/' \
                         'ifcfg-{}'.format(nic_name)
            if os.path.exists(ifcfg_path):
                os.remove(ifcfg_path)
            for mac in dump_config:
                if nic_mac == mac:
                    for num, item in enumerate(dump_config[mac]):
                        if 'DEVICE' in item:
                            dump_config[mac][num] = 'DEVICE={}' \
                                                    '\n'.format(nic_name)
                    with open(ifcfg_path, 'w') as ifcfg_file:
                        ifcfg_file.writelines(dump_config[mac])
    logger.info('Network interfaces has been restored.')


def get_tag():
    """Check "tag" existence for the interface

    The "tag" that should be tied to the port will be used to set the
    requested default route.
    """
    try:
        config_drive = execute_shell_command('mount /dev/sr0 /mnt')
        logger.info(config_drive)
        with open('/mnt/openstack/latest/meta_data.json') as json_file:
            metadata = json.load(json_file)
        execute_shell_command('umount /mnt')
        logger.info('Config drive unmounted')
    except ValueError:
        logger.info('Unable to mount config drive or fetch metadata.')

    metadata = metadata['devices']
    for meta in metadata:
        if 'tags' in meta:
            tags = meta['tags']
            for tag in tags:
                if args.tag == tag:
                    logger.info('The tag "{}" for mac "{}" found'.
                                format(tag, meta['mac']))
                    meta_data = {'mac': meta['mac'], 'tag': tag}
                    return meta_data
    return None


def set_default_route(nics, tag):
    """Set default route for instance based on tag

    The route set to the ifcfg file with the lower metric.
    This is done to ensure that the requested route always will get priority.
    """
    for nic in nics:
        for nic_name, nic_mac in iter(nic.items()):
            if nic_mac == tag['mac'] and tag['tag'] == args.tag:
                ifcfg_path = '/etc/sysconfig/network-scripts/' \
                             'ifcfg-{}'.format(nic_name)
                with open(ifcfg_path, 'a') as ifcfg_file:
                    ifcfg_file.write('IPV4_ROUTE_METRIC=10')
                logger.info('The metric "10" has been set to {} '
                            'interface.'.format(nic_name))


def execute_shell_command(cmd):
    """Execute shell command

    The subprocess.check_output executes command provided as list.
    If the command will be provided as string, it will be converted to list
    and then executed.
    """
    if not isinstance(cmd, list):
        cmd = cmd.split()
    try:
        logger.info('Execute command: {}'.format(cmd))
        output = subprocess.check_output(cmd)
    except subprocess.CalledProcessError as e:
        logger.info('Command failed: {}'.format(e))
        raise
    return output


def main():
    logger.info('Start network configuration')
    if not cloud_init_installed():
        raise ValueError('The instance must have cloud-init installed to'
                         'properly configure the networking.')
    configure_ssh_dns()
    ifaces = check_existing_interfaces()
    if args.init or not os.path.exists(args.nics_data_path):
        logger.info('Check for port tag')
        tag = get_tag()
        logger.info('Verify interfaces config')
        ifaces = verify_interfaces_config(ifaces, tag)
        logger.info('Dump interfaces config')
        dump_interfaces_config(ifaces, tag)
        if tag:
            set_default_route(ifaces, tag)
    else:
        logger.info('Recreate network configuration')
        recreate_interfaces_config(args.nics_data_path, ifaces)
    execute_shell_command('systemctl restart network')
    logger.info('Network configuration completed')


if __name__ == '__main__':
    main()



# Copyright 2020 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from nfv_tempest_plugin.tests.common import shell_utilities as shell_utils
from nfv_tempest_plugin.tests.scenario import base_test
from oslo_log import log as logging
from tempest import config

CONF = config.CONF
LOG = logging.getLogger('{} [-] nfv_plugin_test'.format(__name__))


class TestHciScenarios(base_test.BaseTest):
    def __init__(self, *args, **kwargs):
        super(TestHciScenarios, self).__init__(*args, **kwargs)
        self.hypervisor_ip = None
        self.image_ref = CONF.compute.image_ref

    def setUp(self):
        """Set up a single tenant with an accessible server.

        If multi-host is enabled, save created server uuids.
        """
        super(TestHciScenarios, self).setUp()
        # pre setup creations and checks read from config files

    def test_volume_in_hci_nfv_setup(self, test='nfv_hci_basic_volume'):
        """Test attaches the volume to the instance and writes it.

        Also writing the content into the instance volume.

        :param test: Test name from the config file
        """
        servers, key_pair = self.create_and_verify_resources(test=test)
        volume_id = self.create_volume()
        attachment = self.attach_volume(servers[0], volume_id)
        self.assertIn('device', attachment)
        ssh_source = self.get_remote_client(servers[0]['fip'],
                                            username=self.instance_user,
                                            private_key=key_pair[
                                                'private_key'])
        LOG.info('Execute write test command')
        out = ssh_source.exec_command(
            'sudo dd if=/dev/zero of=/dev/vdb bs=4096k count=256 oflag=direct')
        self.assertEmpty(out)
        LOG.info('The {} test passed.'.format(test))

    def test_boot_instance_with_volume_in_hci_nfv_setup(
            self, test='nfv_hci_instance_volume'):
        """Test creates a instance with the volume and writes it.

        Also writing the content into the instance volume.

        :param test: Test name from the config file
        """
        volume = self.create_volume()
        block_device_mapping = [{'device_name': 'vdb',
                                 'volume_id': volume['id'],
                                 'delete_on_termination': False}]
        servers, key_pair = self.create_and_verify_resources(
            test=test, block_device_mapping=block_device_mapping)
        ssh_source = self.get_remote_client(servers[0]['fip'],
                                            username=self.instance_user,
                                            private_key=key_pair[
                                                'private_key'])
        LOG.info('Execute write test command')
        out = ssh_source.exec_command(
            'sudo dd if=/dev/zero of=/dev/vdb bs=4096k count=256 oflag=direct')
        self.assertEmpty(out)
        LOG.info('The {} test passed.'.format(test))

    def test_volume_using_img_in_hci_nfv_setup(
            self, test='nfv_hci_image_volume'):
        """Test attaches the volume to the instance and writes it.

        Also writing the content into the instance volume.

        :param test: Test name from the config file
        """
        servers, key_pair = self.create_and_verify_resources(test=test)
        volume_id = self.create_volume(imageRef=self.image_ref)
        attachment = self.attach_volume(servers[0], volume_id)
        self.assertIn('device', attachment)
        ssh_source = self.get_remote_client(servers[0]['fip'],
                                            username=self.instance_user,
                                            private_key=key_pair[
                                                'private_key'])
        LOG.info('Execute write test command')
        out = ssh_source.exec_command(
            'sudo dd if=/dev/zero of=/dev/vdb bs=4096k count=256 oflag=direct')
        self.assertEmpty(out)
        LOG.info('The {} test passed.'.format(test))

    def test_ceph_health_status_in_hci_nfv_setup(
            self, test='nfv_hci_ceph_health'):
        """Test ceph health status.

        :param test: Test name from the config file
        """
        LOG.info('Execute ceph health status test command')
        hyper_kwargs = {'shell': CONF.nfv_plugin_options.undercloud_rc_file}
        controller_ip = shell_utils.\
            get_controllers_ip_from_undercloud(**hyper_kwargs)[0]
        cmd = "sudo docker exec ceph-mon-`hostname` ceph -s | grep health | "\
              "cut -d':' -f2 | sed 's/^[ \t]*//;s/[ \t]*$//'"
        result = shell_utils.\
            run_command_over_ssh(controller_ip, cmd).replace("\n", "")
        self.assertEqual(result, 'HEALTH_OK')
        LOG.info('The {} test passed.'.format(test))



#!/bin/env python
# Copyright 2019 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# The script is able to install requested pip packages.

import argparse
import logging
import subprocess
import sys

logging.basicConfig(filename='/var/log/messages', filemode='a',
                    format='%(asctime)s %(name)s %(levelname)s: %(message)s',
                    datefmt="%h %d %H:%M:%S", level=logging.INFO)
handler = logging.StreamHandler(sys.stdout)
logger = logging.getLogger("PackagesDeploy")
logger.addHandler(handler)


def execute_shell_command(cmd):
    """Execute shell command

    The subprocess.check_output executes command provided as list.
    If the command will be provided as string, it will be converted to list
    and then executed.
    """
    if not isinstance(cmd, list):
        cmd = cmd.split()
    try:
        logger.info('Execute command: {}'.format(cmd))
        output = subprocess.check_output(cmd)
    except subprocess.CalledProcessError as e:
        logger.info('Command failed: {}'.format(e))
        raise
    return output


def install_pip():
    """Installing pip"""
    logger.info('Checking for pip installed')
    try:
        import pip  # noqa
        logger.info('Pip was found')
    except ImportError:
        logger.info('Pip was not found. Installing.')
        execute_shell_command('curl https://bootstrap.pypa.io/get-pip.py'
                              ' -o /tmp/get-pip.py')
        execute_shell_command('python /tmp/get-pip.py')
        logger.info('Update pip and setuptools to the latest version')
        execute_shell_command('pip install -U pip setuptools')


def install_pip_packages(packages=None):
    """Install pip provided package/s"""
    if packages is None:
        raise ValueError('The pip install has been called, '
                         'but no packages provided.')
    packages_list = packages.split(',')
    packages = ' '.join(packages_list)
    logger.info('Installing {} packages'.format(packages_list))
    execute_shell_command('pip install {}'.format(packages))


def main():
    logger.info('Start package deploy script configuration')
    parser = argparse.ArgumentParser(description='Bootstraping test config '
                                                 'script')
    parser.add_argument('--pip-packages',
                        help='Packages that should be installed by pip. '
                             'Separate multiple packages by comma.',
                        required=False)
    args = parser.parse_args()

    if args.pip_packages:
        install_pip()
        install_pip_packages(args.pip_packages)
    logger.info('Package deploy script configuration completed')


if __name__ == '__main__':
    main()



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

from enum import IntEnum
from math import ceil
from nfv_tempest_plugin.tests.common import shell_utilities as shell_utils
from oslo_log import log
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils


CONF = config.CONF
LOG = log.getLogger('{} [-] nfv_plugin_test'.format(__name__))


class QoSManagerMixin(object):

    class Servers(IntEnum):
        CLIENT_1 = 0
        CLIENT_2 = 1
        SERVER = 2

    # Unit conversion from iperf report to os qos units [10^6]
    KBYTES_TO_MBITS = 10 ** 6
    LOG_5102 = "/tmp/listen-5102.txt"
    LOG_5101 = "/tmp/listen-5101.txt"

    def create_network_qos_policy(self, namestart='qos-policy'):
        """Creates a network QoS policy"""
        qos_client = self.os_admin.qos_client
        result = qos_client.create_qos_policy(
            name=data_utils.rand_name(namestart))
        self.assertIsNotNone(result, 'Unable to create policy')
        qos_policy = result['policy']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        qos_client.delete_qos_policy,
                        qos_policy['id'])
        return qos_policy

    def create_min_bw_qos_rule(self, policy_id=None, min_kbps=None,
                               direction='egress'):
        """Creates a minimum bandwidth QoS rule

        NOTE: Not all kernel versions support minimum bandwidth for all
        NIC drivers.

        Only egress (guest --> outside) traffic is currently supported.

        :param policy_id
        :param min_kbps: Minimum kbps bandwidth to apply to rule
        :param direction: Traffic direction that the rule applies to
        """
        SUPPORTED_DIRECTIONS = 'egress'
        if not policy_id:
            self.assertNotEmpty(self.qos_policy_groups,
                                'Unable to create_min_bw_qos_rule '
                                'self.qos_policy_groups is Empty')
            policy_id = self.qos_policy_groups[0]['id']
        if direction not in SUPPORTED_DIRECTIONS:
            raise ValueError('{d} is not a supported direction, supported '
                             'directions: {s_p}'
                             .format(d=direction,
                                     s_p=SUPPORTED_DIRECTIONS.join(', ')))
        qos_min_bw_client = self.os_admin.qos_min_bw_client
        result = qos_min_bw_client.create_minimum_bandwidth_rule(
            policy_id, **{'min_kbps': min_kbps, 'direction': direction})
        self.assertIsNotNone(result, 'Unable to create minimum bandwidth '
                                     'QoS rule')
        qos_rule = result['minimum_bandwidth_rule']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            qos_min_bw_client.delete_minimum_bandwidth_rule, policy_id,
            qos_rule['id'])

    def create_max_bw_qos_rule(self, policy_id=None, max_kbps=None,
                               max_burst_kbps=None,
                               direction='egress'):
        """Creates a maximum bandwidth QoS rule

        Only egress (guest --> outside) traffic is currently supported.

        :param policy_id
        :param max_kbps: Maximum kbps bandwidth to apply to rule
        :param max_burst_kbps: max burst bandwidth to apply to rule
        :param direction: Traffic direction that the rule applies to
        """

        SUPPORTED_DIRECTIONS = 'egress'
        if not policy_id:
            self.assertNotEmpty(self.qos_policy_groups,
                                'Unable to create_max_bw_qos_rule '
                                'self.qos_policy_groups is Empty')
            policy_id = self.qos_policy_groups[0]['id']
        if direction not in SUPPORTED_DIRECTIONS:
            raise ValueError('{d} is not a supported direction, supported '
                             'directions: {s_p}'
                             .format(d=direction,
                                     s_p=SUPPORTED_DIRECTIONS.join(', ')))
        bw_rules = {'direction': direction,
                    'max_kbps': max_kbps,
                    'max_burst_kbps': max_burst_kbps}
        qos_max_bw_client = self.os_admin_v2.network_client_v2
        result = qos_max_bw_client.create_bandwidth_limit_rule(policy_id,
                                                               **bw_rules)
        self.assertIsNotNone(result, 'Unable to create maximum bandwidth '
                                     'QoS rule')
        qos_rule = result['bandwidth_limit_rule']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            qos_max_bw_client.delete_bandwidth_limit_rule, policy_id,
            qos_rule['id'])

    def create_qos_policy_with_rules(self, use_default=True, **kwargs):
        """Create_qos_policy with rules

        :param kwargs:
            kwargs['max_kbps']: parameter indicates maximum qos requested
            kwargs['min_kbps']: parameter indicates minimum qos requested
        :param use_default: parameter use self.qos_policy_groups for the test
        """
        qos_policy_groups = \
            self.create_network_qos_policy()
        if use_default:
            self.qos_policy_groups = qos_policy_groups
            self.assertNotEmpty(self.qos_policy_groups,
                                'Unable to qos policy '
                                'self.qos_policy_groups is Empty')
        if 'min_kbps' in kwargs:
            self.create_min_bw_qos_rule(
                policy_id=qos_policy_groups['id'],
                min_kbps=kwargs['min_kbps'])
            kwargs.pop('min_kbps')
        if 'max_kbps' in kwargs:
            self.create_max_bw_qos_rule(
                policy_id=qos_policy_groups['id'],
                **kwargs)
        return qos_policy_groups

    def check_qos_attached_to_guest(self, server, min_bw=False):
        """Check QoS attachment to guest

        This method checks if QoS is applied to an interface on hypervisor
        that is attached to guest

        :param server
        :param min_bw: Check for minimum bandwidth QoS
        """
        # Initialize parameters
        found_qos = False
        interface_data = shell_utils. \
            get_interfaces_from_overcloud_node(server['hypervisor_ip'])
        ports_client = self.os_admin.ports_client
        ports = ports_client.list_ports(device_id=server['id'])
        # Iterate over ports
        for port in ports['ports']:
            # If port has a QoS policy
            if port['qos_policy_id']:
                found_qos = True
                # Construct regular expression to locate port's MAC address
                re_string = r'.*{}.*'.format(port['mac_address'])
                line = re.search(re_string, interface_data)
                # Failed to locate MAC address on hypervisor
                if not line:
                    raise ValueError("Failed to locate interface with MAC "
                                     "'{}' on hypervisor"
                                     .format(port['mac_address']))
                line = line.group(0)
                # Check minimum bandwidth QoS
                if min_bw:
                    qos_min_bw_client = self.os_admin.qos_min_bw_client
                    min_qos_rule = \
                        qos_min_bw_client.list_minimum_bandwidth_rules(
                            port['qos_policy_id'])['minimum_bandwidth_rules']
                    # OpenStack API displays the size in Kbps
                    min_kbps = min_qos_rule[0]['min_kbps']
                    # Construct string to match Linux operating system
                    min_mbps = str(int(ceil(min_kbps / 1000)))
                    min_mbps = '{}Mbps'.format(min_mbps)
                    # Linux operating system displays the size in Mbps
                    qos = re.search(r'min_tx_rate \w+', line)
                    # Failed to locate min QoS
                    if not qos:
                        raise ValueError("Failed to dicover min QoS for "
                                         "interface with MAC '{}'"
                                         .format(port['mac_address']))
                    qos = qos.group(0)
                    # Filter QoS number
                    qos = qos.replace('min_tx_rate ', '')
                    self.assertEqual(min_mbps, qos)
        if not found_qos:
            raise ValueError('No QoS policies were applied to ports')

    def run_iperf_test(self, qos_policies=[], servers=[], key_pair=[]):
        """run_iperf_test

        This method receive server list, and prepare machines for iperf
        and run test commands in server and clients

        :param qos_policies qos policy created for servers
        :param servers servers list, at least 3
        :param key_pair servers key pairs
        """
        srv = QoSManagerMixin.Servers
        servers_ports_map = \
            [self.os_admin.ports_client.list_ports(
                device_id=server['id']) for server in self.servers]
        # Find machines direct ports
        tested_ports = [shell_utils.find_vm_interface(
            ports, vnic_type='direct') for ports in servers_ports_map]
        # Bind to iperf server ip_addr
        ip_addr = tested_ports[srv.SERVER][1]
        # Set pors with QoS
        LOG.info('Update client ports with QoS policies...')
        # Assume server is the last server index 2,
        # In case one policy parsed, CLIENT_1 is the tested port
        [self.update_port(
            tested_ports[i][0],
            **{'qos_policy_id': qos_policies[i]['id']})
            for i in range((len(self.servers) - 1)
                           and len(qos_policies))]
        LOG.info('Run iperf server on server3...')
        ssh_dest = self.get_remote_client(servers[srv.SERVER]['fip'],
                                          username=self.instance_user,
                                          private_key=key_pair[
                                          'private_key'])
        server_command = "sudo yum install iperf -y; "
        log_5102 = QoSManagerMixin.LOG_5102
        log_5101 = QoSManagerMixin.LOG_5101
        server_command += \
            "(nohup iperf -s -B {} -p 5102 -i 10 > {} 2>&1)& ".format(
                ip_addr, log_5102)
        server_command += \
            "(nohup iperf -s -B {} -p 5101 -i 10 > {} 2>&1)& ".format(
                ip_addr, log_5101)
        LOG.info('Receive iperf traffic from Server3...')
        ssh_dest.exec_command(server_command)
        # Running vm CLIENT_1 last since it is tested VM
        ssh_source1 = self.\
            get_remote_client(servers[srv.CLIENT_1]['fip'],
                              username=self.instance_user,
                              private_key=key_pair['private_key'])
        LOG.info('Send iperf traffic from Server1...')
        client_command1 = "sudo yum install iperf -y"
        ssh_source1.exec_command(client_command1)
        ssh_source2 = self.\
            get_remote_client(servers[srv.CLIENT_2]['fip'],
                              username=self.instance_user,
                              private_key=key_pair['private_key'])
        LOG.info('Send iperf traffic from Server2...')
        client_command2 = "sudo yum install iperf -y"
        # must run in background, to check guaranteed min/bw for client1
        # Running vm CLIENT_2 initially, since it serves as one helper
        # nohup iperf -c 40.0.0.130 -T s2 -p 5101 -t  2>&1 &
        # (nohup iperf -c 40.0.0.164 -T s2 -p 5101 -t 60 >
        # /tmp/iperf.out 2>&1)&"
        ssh_source2.exec_command(client_command2)
        client_command2 = "nohup iperf -c {} -T s2 -p {} -t 120 >" \
                          "/tmp/iperf.out 2>&1 &".format(ip_addr, '5102')
        ssh_source2.exec_command(client_command2)
        # vm with best-effort quality in case of one policy parsed
        client_command1 = \
            "iperf -c {} -T s1 -p {} -t 120".format(ip_addr, '5101')
        ssh_source1.exec_command(client_command1)

    def collect_iperf_results(self, qos_rules_list=[],
                              servers=[], key_pair=[]):
        """collect_iperf_results

        This method receive server list, ssh to iperf server and collect
        iperf reports and compare it to QoS rules created for clients ports

        :param qos_rules_list qos rules list fpr tested ports
        :param servers servers list, at least 3
        :param key_pair servers key pairs
        """

        srv = QoSManagerMixin.Servers
        kbytes_to_mbits = QoSManagerMixin.KBYTES_TO_MBITS
        log_files = [QoSManagerMixin.LOG_5101, QoSManagerMixin.LOG_5102]
        # Run grep command over iperf server to verify BW is OK
        # File format is as following
        # [  4]  0.0-10.0 sec  4.76 GBytes  4.09 Gbits/sec
        # [  4] 10.0-20.0 sec  4.76 GBytes  4.09 Gbits/sec
        # cat /tmp/listen-5101.txt | while read line ; do
        # if [[ $line =~ [[:space:]]([0-9]{1,2}.[0-9]{1,2})[[:space:]]Gbits ]];
        # then echo ${BASH_REMATCH[1]}; fi ; done
        LOG.info('Collect iperf logs from iperf server, server3...')
        command = r"cat {} | while read line ;do  "
        command += r"if [[ \"$line\" =~ [[:space:]]"
        command += r"([0-9]{1,2}\.[0-9]{1,2})[[:space:]]Gbits ]]; "
        command += r"then echo \"${BASH_REMATCH[1]}\"; fi; done| sort| uniq"
        # Recive result with number
        ssh_dest = self.get_remote_client(servers[srv.SERVER]['fip'],
                                          username=self.instance_user,
                                          private_key=key_pair[
                                          'private_key'])
        # Assert result
        for index in range(srv.SERVER):
            # If Default QoS, such as min_bw check only srv.CLIENT_1
            if len(self.qos_policy_groups) > 0 and index == srv.CLIENT_2:
                break
            out_testing = ssh_dest.\
                exec_command(command.replace('{}', log_files[index]))
            iperf_rep = \
                [i for i in (
                    out_testing.encode('utf8')).split() if i != '']
            self.assertNotEmpty(
                iperf_rep,
                "Please check QoS definitions, iperf result for in file {}"
                " is empty or low".format(log_files[index]))
            # Apply average for result
            res = [float(i.decode("utf-8").
                         replace('"', '')) for i in iperf_rep]
            rep = sum(res) / len(res)
            qos_type = 'max_kbps'
            # In case of min_bw only one policy is set
            if 'min_kbps' in qos_rules_list[srv(index)]:
                qos_type = 'min_kbps'
            dev = abs(rep * kbytes_to_mbits
                      / qos_rules_list[srv(index)][qos_type] - 1)
            # Check if only one policy to Verify
            self.assertLess(dev, 0.08,
                            "iperf result deviates more than 0.08")



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import base64
import json
import paramiko
import re
import shlex
import subprocess as sp

from backports.configparser import ConfigParser
from collections import OrderedDict
from oslo_log import log
from oslo_serialization import jsonutils
from tempest import config
"""Python 2 and 3 support"""
from six.moves import StringIO

CONF = config.CONF
LOG = log.getLogger('{} [-] nfv_plugin_test'.format(__name__))


def run_command_over_ssh(host, command):
    """This Method run Command Over SSH

    Provide Host, user and pass into configuration file

    :param host
    :param command
    """

    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())

    """Assuming all check done in Setup,
    otherwise Assert failing the test
    """
    if CONF.nfv_plugin_options.overcloud_node_pkey_file_rsa:
        ssh.connect(host,
                    username=CONF.nfv_plugin_options.overcloud_node_user,
                    pkey=CONF.nfv_plugin_options.
                    overcloud_node_pkey_file_rsa)
    else:
        ssh.connect(host,
                    username=CONF.nfv_plugin_options.overcloud_node_user,
                    password=CONF.nfv_plugin_options.overcloud_node_pass)

    LOG.info("Executing command: %s" % command)
    stdin, stdout, stderr = ssh.exec_command(command)
    """
    In python3 the result returned is in bytes instead of literal
    We want to convert it to unicode
    """
    result = stdout.read().decode('UTF-8')
    ssh.close()
    return result


def get_interfaces_from_overcloud_node(node_ip, cmd=None):
    """Retrieve interfaces from overcloud node

    :param node_ip:
    :param cmd:
    list member and return an array

    :return output:
    """
    if not cmd:
        cmd = 'sudo ip link show'
    output = run_command_over_ssh(node_ip, cmd)
    return output


def retrieve_content_from_files(node, files):
    """Retrieve encoded base64 content from files on Linux hosts

    Using a single SSH connection that executes a bash for loop that
    iterates over files to construct a JSON containing file's name and
    file's encoded content.

    The retrieved content will be decoded from base64.

    If file doesn't exist/can not be parsed, the content will be equal
    to 'None'.

    :param node: Which node to retrive content from
    :param files: List of files to retrive content from

    :return file_content
    """

    bash_list = ' '.join(files)
    file_content = {}
    # Create JSONs for each file with base64 encoded content
    # if file doesn't exist/can't be decodded returns 'None'
    cmd = '''
          function construct_json() {{
              echo "{{\\"$1\\": \\"$2\\"}}"
          }}

          a=({list})
          for file in ${{a[@]}}; do
              content="None"
              if [[ -f $file ]]; then
                      output=$(sudo base64 $file -w 0)
                      if [[ $output ]];then
                          content=$output
                     fi
              fi
              construct_json $file $content
          done
          '''.format(list=bash_list)

    guest_content = run_command_over_ssh(node, cmd).split('\n')
    # Output will always produce an additional unnecessary new line
    del guest_content[-1]
    # Parse and construct a JSON containing all the results
    for result in guest_content:
        file_content.update(jsonutils.loads(result))
    # Decode content from base64
    for content in file_content:
        if file_content[content] != 'None':
            parsed_content = base64.b64decode(file_content[content])
            file_content[content] = parsed_content.split('\n')

    return file_content


def retrieve_content_from_hiera(node, keys,
                                hiera_file='/etc/puppet/hiera.yaml'):
    """Get configuration values using hiera tool

    :param node: The node to retrieve the value from
    :param keys: The keys that should be provided to retrieve the value
                 Multiple keys should be provided as the array
    :param hiera_file: Hiera config file
    :return: List of the values returned
    """
    hiera_template = 'sudo hiera -c {hiera_file} {key};'
    hiera_command = ''
    for key in keys:
        hiera_command += hiera_template.format(hiera_file=hiera_file,
                                               key=key)
    hiera_content = run_command_over_ssh(node, hiera_command)
    hiera_content = hiera_content.replace(',\n', ',').strip().split('\n')
    return hiera_content


def parse_int_ranges_from_number_string(input_string):
    """Parses integers from number string

    :param input_string
    """
    # Assign helper variable
    parsed_input = []
    # Construct a list of integers from given number string,range
    for cell in input_string.split(','):
        if '-' in cell:
            start, end = cell.split('-')
            parsed_range = list(range(int(start), int(end) + 1))
            parsed_input.extend(parsed_range)
        else:
            parsed_input.append(int(cell))
    return parsed_input


def get_controllers_ip_from_undercloud(**kwargs):
    """This method returns the list of controllers ip

    :param kwargs['shell']
    """
    command = 'openstack server list -c \'Name\' -c ' \
              '\'Networks\' -f value | grep -i {0} | ' \
              'cut -d\"=\" -f2'.format('controller')
    ip_address_list = run_local_cmd_shell_with_venv(
        command, kwargs['shell'])
    return ip_address_list


def get_overcloud_config(overcloud_node, config_path):
    """Get overcloud configuration

    The method will get the config file by the provided path from the
    overcloud node specified by the user.
    The path could lead to the regular config path or to the
    containerized bind.

    :param overcloud_node: Which server to get config from
    :param config_path: The path of the configuration file

    :return config_data
    """

    get_config_data = 'sudo cat {0}'.format(config_path)
    config_data = run_command_over_ssh(overcloud_node,
                                       get_config_data)

    return config_data


def get_value_from_ini_config(overcloud_node, config_path,
                              check_section, check_value,
                              multi_key_values=False):
    """Get value from INI configuration file

    :param overcloud_node:   The node that config should be pulled from
    :param config_path:      The path of the configuration file
    :param check_section:    Section within the config
    :param check_value:      Value that should be checked within the config
                             The variable could hold multiple values separated
                             by comma.
    :param multi_key_values: Flag on request to hold multiple values for
                             single key from ini file
    :return return_value
    """

    class M(OrderedDict):
        def __setitem__(self, key, value):
            v_val = self.get(key)
            if v_val is not None and type(value) == list:
                v_val.append(value[0])
            else:
                v_val = value
            # still using python2.7 super, for backport portability
            super(M, self).__setitem__(key, v_val)

    ini_config = get_overcloud_config(overcloud_node, config_path)
    config_parser_args = {'allow_no_value': True}
    if multi_key_values:
        config_parser_args['dict_type'] = M
    config_parser_args['strict'] = False
    get_value = ConfigParser(**config_parser_args)
    get_value.read_file(StringIO(ini_config))
    value_data = []
    for value in check_value.split(','):
        value_data.append(get_value.get(check_section, value))

    return ','.join(value_data)


def run_local_cmd_shell_with_venv(command, shell_file_to_exec=None):
    """This Method runs command on tester local host

    Shell_file_to_exec path to source file default is None
    TBD: Add support to return, hosts list
    TBD: Return None in case no aggregation found.

    :param command
    :param shell_file_to_exec
    """
    if command == '':
        raise ValueError("command parameter is empty")
    if shell_file_to_exec is not None:
        source = 'source %s' % shell_file_to_exec
        pipe = sp.Popen(['/bin/bash', '-c', '%s && %s' % (
            source, command)], stdout=sp.PIPE)
    else:
        pipe = sp.Popen(['/bin/bash', '-c', '%s' % command],
                        stdout=sp.PIPE)
    """
    In python3 the result returned is in bytes instead of literal
    We want to convert it to unicode
    """
    result = pipe.stdout.read().decode('UTF-8')
    return result.split()


def check_pid_ovs(ip_address):
    """This method checks if ovs pid exist

    param ip_address: server ip address
    return  ovs pid or Exception if it does not exist
    """

    ovs_process = "sudo pidof ovs-vswitchd"
    ovs_process_pid = (run_command_over_ssh(ip_address,
                                            ovs_process))\
        .strip('\n')
    if not ovs_process_pid:
        raise ValueError('The ovs-vswitchd process is missing.')
    return ovs_process_pid


def check_guest_interface_config(ssh_client, provider_networks,
                                 hostname):
    """Check guest inteface network configuration

    The function aims to check if all provider networks are configured
    on guest operating system.

    :param ssh_client: SSH client configured to connect to server
    :param provider_networks: Server's provider networks details
    """
    for provider_network in provider_networks:
        mac = provider_network['mac_address']
        ip = provider_network['ip_address']
        # Attempt to discover guest interface using a MAC address
        guest_interface = ssh_client.get_nic_name_by_mac(mac)
        msg = ("Guest '{h}' has no interface with mac '{m}")
        assert guest_interface, msg.format(h=hostname,
                                           m=mac)
        LOG.info("Located '{m}' in guest '{h}' on interface '{g}'".format(
            m=mac, h=hostname, g=guest_interface))
        # Attempt to discover guest interface using an IP address
        ip_interface = ssh_client.get_nic_name_by_ip(ip)
        msg = ("Guest '{h}' exepected to have interface '{g}' to be "
               "configured with IP address '{i}'")
        assert ip_interface, msg.format(h=hostname, g=guest_interface,
                                        i=ip)
        LOG.info("Guest '{h}' has interface '{g}' configured with "
                 "IP address '{i}".format(h=hostname, g=guest_interface,
                                          i=ip))


def run_hypervisor_command_build_from_config(file_path, search_param,
                                             servers_ips,
                                             multi_key_values,
                                             command, filter_regexp=None):
    """buildandrun_hypervisor_command_from_config

    The method checks for a value [search_param] in controller/compute
    ini_file [file_path], build cli [command] include ini_param and run,
     on specific compute,
    the results are [filter_regexp] and returned as dict

    filtered by desired regexp [nic_req_status] res_dict returned for test.

    :param file_path:        Configuration ini file path
    :param search_param:     Search section-value in ini file
    :param servers_ips:      Host ip addresses hosting files and devices
    :param filter_regexp:    Regular expresion to search on filtered_command
    :param multi_key_values: Flag on request to hold multiple values for
                             single key
    :param command:          hypervisor command to be run regexp on
    :return res_dict
    """
    res_dict = {}
    for hypervisor in servers_ips:
        if hypervisor not in res_dict:
            res_dict[hypervisor] = []
        result = get_value_from_ini_config(hypervisor,
                                           file_path,
                                           search_param['section'],
                                           search_param['value'],
                                           multi_key_values)
        msg = "No {} found in".format(search_param)
        assert result != '', "{} {}".format(msg, hypervisor)

        result = "[" + result.replace('\n', ", ") + "]"
        dev_names = [x.get('devname') for x in json.loads(result)]
        # The nic-partitioning deployment does not store "devname" param.
        # It has the "domain", "function" and other attributes.
        # As a result, many None params added to the list.
        # The below command excludes the "None" from the list.
        dev_names = [devname for devname in dev_names if devname]
        cmd = ''
        for device in dev_names:
            cmd += "{} {};".format(command, device)
        result = \
            get_interfaces_from_overcloud_node(hypervisor, cmd)

        for line in result.split("\n"):
            nic_stat = line if not filter_regexp else re.\
                findall(filter_regexp, line)
            if len(nic_stat) > 0:
                res_dict[hypervisor].append(nic_stat[0])

    return res_dict


def find_vm_interface(ports=[],
                      vnic_type='normal'):
    """find vm interface

    The function receive port list and search for requested
    vnic_type.

    :param ports: ports connected to specific server
    :param vnic_type: vnic_type nomal/direct/direct_physical

    return port_id, ip_address
    """
    assert len(ports), 'ports is empty or None'
    return [[port['id'], port['fixed_ips'][0]['ip_address']]
            for port in ports['ports']
            if port['binding:vnic_type'] == vnic_type][0]


def continuous_ping(ip_dest,
                    mtu=1422,
                    duration=10):
    """continuous_ping

    The function send ping command in background mode
    from tempest host to fip vm

    :param ip_dest: comma separated ip dests
    :param mtu: ping mtu size to check
    :param duration: duration of ping test.
    """
    cmd = "nohup ping -i 1 -c {duration} -q " \
          "-s {mtu} {ip_dest}"
    msg = "no ping dest '{h}' "
    assert ip_dest, msg.format(h=ip_dest)
    ip_list = ip_dest.split(",")
    for ping_ip in ip_list:
        cmd_line = cmd.format(mtu=mtu, duration=duration, ip_dest=ping_ip)
        log_file = "/tmp/ping-{ip_dest}.txt".format(ip_dest=ping_ip)
        with open(log_file, 'w') as out:
            sp.Popen(shlex.split(cmd_line), stdout=out,
                     stderr=out)
        LOG.debug('pinging %(ping_ip)s for %(duration)s ')


def stop_continuous_ping():
    """stop continuous_ping

    The function does pgrep ping on local machines, send kill with
    stop statistics
    kill -SIGINT `pgrep ping
    """
    cmd = "pgrep ping"
    remote_command = shlex.split(cmd)
    pipe = sp.Popen(remote_command, stdout=sp.PIPE)
    ping_pid = pipe.stdout.read().decode('UTF-8')\
        .rstrip('\n').replace('\n', ' ')
    LOG.info("Found the following pids: {}".format(ping_pid))
    if ping_pid != '':
        cmd = "/bin/kill -SIGINT {}".format(ping_pid)
        remote_command = shlex.split(cmd)
        pipe = sp.Popen(remote_command, stdout=sp.PIPE)
        LOG.info("Ping process termiated {}".
                 format(pipe.stdout.read().decode('UTF-8').rstrip('\n')))



# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from nfv_tempest_plugin.tests.scenario import base_test
from oslo_log import log as logging
from tempest import config

CONF = config.CONF
LOG = logging.getLogger('{} [-] nfv_plugin_test'.format(__name__))


class TestDpdkScenarios(base_test.BaseTest):
    def __init__(self, *args, **kwargs):
        super(TestDpdkScenarios, self).__init__(*args, **kwargs)
        self.instance = None
        self.image_ref = CONF.compute.image_ref
        self.flavor_ref = CONF.compute.flavor_ref
        self.public_network = CONF.network.public_network_id
        self.maxqueues = None

    def setUp(self):
        """Set up a single tenant with an accessible server

        If multi-host is enabled, save created server uuids.
        """
        super(TestDpdkScenarios, self).setUp()

    def _test_queue_functionality(self, queues):
        """Checks DPDK queues functionality

        Booting number of instances with various number of cpus based on the
        setup queues number.
        """
        LOG.info('Prepare the queues functionality test')
        self.maxqueues = self.check_number_queues()

        extra_specs = {'extra_specs': {'hw:mem_page_size': str("large"),
                                       'hw:cpu_policy': str("dedicated")}}
        if queues == "min":
            queues = self.maxqueues - 2
        elif queues == "odd":
            queues = self.maxqueues - 1
        elif queues == 'max':
            queues = self.maxqueues + 2
        else:
            queues = self.maxqueues

        LOG.info('Create a flavor for the queues test.')
        queues_flavor = self.create_flavor(name='test-queues', vcpus=queues,
                                           **extra_specs)
        servers, key_pair = \
            self.create_server_with_resources(test='check-multiqueue-func',
                                              flavor=queues_flavor)

        LOG.info('Check connectivity to the queues instance.')
        self.check_instance_connectivity(ip_addr=servers[0]['fip'],
                                         user=self.instance_user,
                                         key_pair=key_pair['private_key'])
        LOG.info('The {} queues test passed.'.format(queues))
        return True

    def test_multicast(self, test='multicast'):
        """The method boots three instances, runs mcast traffic between them"""
        LOG.info('Starting multicast test.')
        servers, key_pair = \
            self.create_server_with_resources(test=test, num_servers=3,
                                              use_mgmt_only=True)
        servers[0]['mcast_srv'] = 'listener1'
        servers[1]['mcast_srv'] = 'listener2'
        servers[2]['mcast_srv'] = 'traffic_runner'
        LOG.info('Listener1 server - {}, '
                 'Listener2 server - {}, '
                 'Traffic_runner server - {}'.format(servers[0]['fip'],
                                                     servers[1]['fip'],
                                                     servers[2]['fip']))
        mcast_group = '224.0.0.1'
        mcast_port = '10000'
        mcast_msg = 'mcast_pass'
        mcast_output = '/tmp/output'
        get_mcast_results = 'cat {}'.format(mcast_output)
        receive_cmd = 'sudo python {0}/multicast_' \
                      'traffic.py -r -g {1} -p {2} -c 1 > {3} ' \
                      '&'.format(self.nfv_scripts_path, mcast_group,
                                 mcast_port, mcast_output)
        send_cmd = 'sudo python {0}/multicast_' \
                   'traffic.py -s -g {1} -p {2} -m {3} -c 1 > {4} ' \
                   '&'.format(self.nfv_scripts_path, mcast_group, mcast_port,
                              mcast_msg, mcast_output)
        for srv in servers:
            LOG.info('Executing multicast script on {} - {}.'
                     .format(srv['mcast_srv'], srv['fip']))
            ssh_source = self.get_remote_client(srv['fip'],
                                                username=self.instance_user,
                                                private_key=key_pair[
                                                    'private_key'])
            ssh_source.exec_command(send_cmd if 'traffic_runner' in
                                                srv['mcast_srv'] else
                                                receive_cmd)

        for receiver in servers:
            if ('listener1' in receiver['mcast_srv']) or \
                    ('listener2' in receiver['mcast_srv']):
                LOG.info('Reading results from {} - {} instance.'
                         .format(receiver['mcast_srv'], receiver['fip']))
                ssh_source = self.get_remote_client(receiver['fip'],
                                                    username=self.
                                                    instance_user,
                                                    private_key=key_pair[
                                                        'private_key'])
                output = ssh_source.exec_command(get_mcast_results)
                results = output.rstrip('\n')
                fail_results_msg = '{} - {} unable to receive multicast ' \
                                   'traffic.'.format(receiver['mcast_srv'],
                                                     receiver['fip'])
                self.assertEqual(results, mcast_msg, fail_results_msg)
                LOG.info('{} - {} received multicast traffic.'
                         .format(receiver['mcast_srv'], receiver['fip']))

        LOG.info('Both listener1 and listener2 received multicast traffic')
        return True

    def test_min_queues_functionality(self):
        msg = "Could not create, ping or ssh to the instance with flavor " \
              "contains vcpus smaller than allowed amount of queues"
        self.assertTrue(self._test_queue_functionality(queues="min"), msg)

    def test_equal_queues_functionality(self):
        msg = "Could not create, ping or ssh to the instance with flavor " \
              "contains vcpus equal to allowed amount of queues"
        self.assertTrue(self._test_queue_functionality(queues="equal"), msg)

    def test_max_queues_functionality(self):
        msg = "Could not create, ping or ssh to the instance with flavor " \
              "contains vcpus max to allowed amount of queues"
        self.assertTrue(self._test_queue_functionality(queues="max"), msg)

    def test_odd_queues_functionality(self):
        msg = "Could not create, ping or ssh to the instance with flavor " \
              "contains odd number of vcpus"
        self.assertTrue(self._test_queue_functionality(queues="odd"), msg)

    def test_rx_tx(self, test='rx_tx'):
        """Test RX/TX on the instance vs nova configuration

        The test compares RX/TX value from the dumpxml of the running
        instance vs values of the overcloud nova configuration
        """

        servers, key_pair = self.create_and_verify_resources(test=test)

        check_section = 'libvirt'
        check_value = 'rx_queue_size,tx_queue_size'
        osp_release = self.get_osp_release()
        if osp_release >= 13:
            config_path = '/var/lib/config-data/puppet-generated/' \
                          'nova_libvirt/etc/nova/nova.conf'
        else:
            config_path = '/etc/nova/nova.conf'

        for srv in servers:
            LOG.info('Test RX/TX for the {} instance'.format(srv['fip']))
            return_value = self.\
                compare_rx_tx_to_overcloud_config(srv, srv['hypervisor_ip'],
                                                  config_path,
                                                  check_section,
                                                  check_value)
            self.assertTrue(return_value, 'The rx_tx test failed. '
                                          'The values of the instance and '
                                          'nova does not match.')
        LOG.info('The {} test passed.'.format(test))



from neutron_tempest_plugin.api import clients as manager
from neutron_tempest_plugin.services.network.json import network_client
from tempest import config

CONF = config.CONF


class Manager(manager.Manager):
    def __init__(self, credentials=None, service=None):
        super(Manager, self).__init__(credentials=credentials)
        self.network_client_v2 = network_client.NetworkClientJSON(
            self.auth_provider,
            CONF.network.catalog_type,
            CONF.network.region or CONF.identity.region,
            endpoint_type=CONF.network.endpoint_type,
            build_interval=CONF.network.build_interval,
            build_timeout=CONF.network.build_timeout,
            **self.default_params)



# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from nfv_tempest_plugin.tests.common import shell_utilities as shell_utils
from nfv_tempest_plugin.tests.scenario import base_test
from oslo_log import log as logging
from tempest.common import waiters
from tempest import config

CONF = config.CONF
LOG = logging.getLogger('{} [-] nfv_plugin_test'.format(__name__))


class TestNfvBasic(base_test.BaseTest):
    def __init__(self, *args, **kwargs):
        super(TestNfvBasic, self).__init__(*args, **kwargs)
        self.hypervisor_ip = None

    def setUp(self):
        """Set up a single tenant with an accessible server.

        If multi-host is enabled, save created server uuids.
        """
        super(TestNfvBasic, self).setUp()
        # pre setup creations and checks read from config files

    def _test_check_package_version(self, test_compute):
        """Check provided packages, service and tuned profile on the compute

        - If given - checks if packages are exist on hypervisor
        - If given - checks if the services are at active state
        - If given - checks the active state of the tuned-profile
        * The test demands test_compute list

        :param test_compute
        """
        self.assertTrue(self.test_setup_dict[test_compute],
                        "test requires check-compute-packages "
                        "list in external_config_file")
        hyper_kwargs = {'shell': CONF.nfv_plugin_options.undercloud_rc_file}
        self.hypervisor_ip = self._get_hypervisor_ip_from_undercloud(
            **hyper_kwargs)[0]
        self.assertNotEmpty(self.hypervisor_ip,
                            "_get_hypervisor_ip_from_undercloud "
                            "returned empty ip list")
        test_result = []
        if 'package-names' in self.test_setup_dict[test_compute]:
            packages = self.test_setup_dict[test_compute]['package-names']
            if packages is not None:
                for package in packages:
                    cmd = "rpm -qa | grep {0}".format(package)
                    result = shell_utils.\
                        run_command_over_ssh(self.hypervisor_ip,
                                             cmd).split()
                    if result:
                        test_result += result

        LOG.info("Found the following packages: %s" % '\n'.join(test_result))
        del test_result[:]

        if 'service-names' in self.test_setup_dict[test_compute]:
            services = self.test_setup_dict[test_compute]['service-names']
            if services is not None:
                for service in services:
                    cmd = "systemctl is-active {0}".format(service)
                    result = shell_utils.\
                        run_command_over_ssh(self.hypervisor_ip,
                                             cmd).strip('\n')
                    if result != 'active':
                        test_result.append("The {0} service is not Active"
                                           .format(service))

        if 'tuned-profile' in self.test_setup_dict[test_compute]:
            tuned = self.test_setup_dict[test_compute]['tuned-profile']
            if tuned is not None:
                cmd = "sudo tuned-adm active | awk '{print $4}'"
                result = shell_utils.run_command_over_ssh(
                    self.hypervisor_ip, cmd).strip('\n')
                if result != tuned:
                    test_result.append("Tuned {0} profile is not Active"
                                       .format(tuned))

        kernel_args = ['nohz', 'nohz_full', 'rcu_nocbs', 'intel_pstate']
        check_grub_cmd = "sudo cat /proc/cmdline"
        result = shell_utils.\
            run_command_over_ssh(self.hypervisor_ip, check_grub_cmd)
        for arg in kernel_args:
            if arg not in result:
                test_result.append("Tuned not set in grub. Need to reboot?")

        test_result = '\n'.join(test_result)
        self.assertEmpty(test_result, test_result)

    def test_mtu_ping_test(self, test='test-ping-mtu', mtu=1973):
        """Test MTU by pinging instance gateway

        The test boots an instance with given args from external_config_file,
        connect to the instance using ssh, and ping with given MTU to GW.
        * This tests depend on MTU configured at running environment.

        :param test: Test name from the config file
        :param mtu: Size of the mtu to check
        """
        # TODO(skramaja): Need to check if it is possible to execute ping
        #                 inside the guest VM using network namespace
        self.assertTrue(self.fip, "Floating IP is required for mtu test")

        servers, key_pair = self.create_and_verify_resources(test=test)

        if 'mtu' in self.test_setup_dict[test]:
            mtu = self.test_setup_dict[test]['mtu']
            LOG.info('Set {} mtu for the test'.format(mtu))

        routers = self.os_admin.routers_client.list_routers()['routers']
        for router in routers:
            if router['external_gateway_info'] is not None:
                gateway = router['external_gateway_info'][
                    'external_fixed_ips'][0]['ip_address']
                break
        else:
            raise ValueError('The gateway of given network does not exists. '
                             'Please assign it and re-run.')

        ssh_source = self.get_remote_client(servers[0]['fip'],
                                            username=self.instance_user,
                                            private_key=key_pair[
                                                'private_key'])
        LOG.info('Execute ping test command')
        out = ssh_source.exec_command('ping -c 1 -M do -s %d %s' % (mtu,
                                                                    gateway))
        msg = "MTU Ping test failed - check your environment settings"
        self.assertTrue(out, msg)
        LOG.info('The {} test passed.'.format(test))

    def test_packages_compute(self):
        self._test_check_package_version("check-compute-packages")

    def test_cold_migration(self, test='cold-migration'):
        """Test cold migration

        The test shuts down the instance, migrates it and brings it up to
        verify resize.
        """
        servers, key_pair = self.create_and_verify_resources(test=test)

        LOG.info('Starting the cold migration.')
        self.os_admin.servers_client. \
            migrate_server(server_id=servers[0]['id'])
        waiters.wait_for_server_status(self.servers_client,
                                       servers[0]['id'], 'VERIFY_RESIZE')
        LOG.info('Confirm instance resize after the cold migration.')
        self.servers_client.confirm_resize_server(server_id=servers[0]['id'])
        LOG.info('Verify instance connectivity after the cold migration.')
        self.check_instance_connectivity(ip_addr=servers[0]['fip'],
                                         user=self.instance_user,
                                         key_pair=key_pair['private_key'])
        succeed = True

        msg = "Cold migration test id failing. Check your environment settings"
        self.assertTrue(succeed, msg)
        LOG.info('The cold migration test passed.')

    def test_emulatorpin(self, test='emulatorpin'):
        """Test emulatorpin on the instance vs nova configuration

        The test compares emulatorpin value from the dumpxml of the running
        instance vs values of the overcloud nova configuration

        Note! - The test suit only for RHOS version 14 and up, since the
                emulatorpin feature was implemented only in version 14.
        """

        servers, key_pair = self.create_and_verify_resources(test=test)

        config_path = '/var/lib/config-data/puppet-generated' \
                      '/nova_libvirt/etc/nova/nova.conf'
        check_section = 'compute'
        check_value = 'cpu_shared_set'

        for srv in servers:
            LOG.info('Test emulatorpin for the {} instance'.format(srv['fip']))
            return_value = self. \
                compare_emulatorpin_to_overcloud_config(srv,
                                                        srv['hypervisor_ip'],
                                                        config_path,
                                                        check_section,
                                                        check_value)
            self.assertTrue(return_value, 'The emulatorpin test failed. '
                                          'The values of the instance and '
                                          'nova does not match.')
        LOG.info('The {} test passed.'.format(test))



# Copyright 2020 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

from nfv_tempest_plugin.tests.scenario import base_test
from oslo_log import log as logging
from tempest import config

CONF = config.CONF
LOG = logging.getLogger('{} [-] nfv_plugin_test'.format(__name__))


class TestLiveMigrationScenarios(base_test.BaseTest):
    def __init__(self, *args, **kwargs):
        super(TestLiveMigrationScenarios, self).__init__(*args, **kwargs)

    def setUp(self):
        """Set up a single tenant with an accessible server.

        If multi-host is enabled, save created server uuids.
        """
        super(TestLiveMigrationScenarios, self).setUp()
        # pre setup creations and checks read from config files

    def _perform_live_migration(self, server, key_pair,
                                use_block_migration=False):
        """Method boots an instance and wait until ACTIVE state

        Migrates the instance to the next available hypervisor.
        """

        host = self.os_admin.servers_client.show_server(
            server['id'])['server']['OS-EXT-SRV-ATTR:hypervisor_hostname']
        self.check_instance_connectivity(ip_addr=server['fip'],
                                         user=self.instance_user,
                                         key_pair=key_pair['private_key'])
        """ Migrate server """
        self.os_admin.servers_client.live_migrate_server(
            server_id=server['id'], block_migration=use_block_migration,
            host=None)
        """ Switch hypervisor id (compute-0 <=> compute-1) """
        count = 1
        if host.find('0') > 0:
            dest = list(host)
            dest[dest.index('0')] = '1'
            dest = ''.join(dest)
        else:
            dest = list(host)
            dest[dest.index('1')] = '0'
            dest = ''.join(dest)
        while count < 30:
            count += 1
            time.sleep(3)
            if dest == self\
                    .os_admin.servers_client.show_server(server[
                    'id'])['server']['OS-EXT-SRV-ATTR:hypervisor_hostname']:
                """Verify connectivity after migration"""
                self.check_instance_connectivity(ip_addr=server['fip'],
                                                 user=self.instance_user,
                                                 key_pair=key_pair[
                                                     'private_key'])
                return True
        return False

    def test_live_migration_block(self, test='test_live_migration_block'):
        """Test live migration with block migration

        Make sure CONF.compute_feature_enabled.live_migration is True
        """
        msg = "live migration is not configured"
        self.assertTrue(CONF.compute_feature_enabled.live_migration, msg)
        msg = "Live migration Failed"
        servers, key_pair = \
            self.create_and_verify_resources(test=test, use_mgmt_only=True)
        self.assertTrue(self._perform_live_migration(
            servers[0], key_pair, use_block_migration=True), msg)

    def test_live_migration_shared(self, test='test_live_migration_shared'):
        """Test live migration with shared storage

        Make sure CONF.compute_feature_enabled.live_migration is True
        """
        msg = "live migration is not configured"
        self.assertTrue(CONF.compute_feature_enabled.live_migration, msg)
        msg = "Live migration Failed"
        servers, key_pair = \
            self.create_and_verify_resources(test=test, use_mgmt_only=True)
        self.assertTrue(self._perform_live_migration(servers[0], key_pair),
                        msg)



# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from nfv_tempest_plugin.tests.common import shell_utilities as shell_utils
from nfv_tempest_plugin.tests.scenario import base_test
from oslo_log import log as logging
from tempest import config

import json
import random
import string

CONF = config.CONF
LOG = logging.getLogger('{} [-] nfv_plugin_test'.format(__name__))


class TestIgmpSnoopingScenarios(base_test.BaseTest):
    def __init__(self, *args, **kwargs):
        super(TestIgmpSnoopingScenarios, self).__init__(*args, **kwargs)

    def setUp(self):
        """Set up a single tenant with an accessible server

        If multi-host is enabled, save created server uuids.
        """
        super(TestIgmpSnoopingScenarios, self).setUp()
        """ pre setup creations and checks read from config files """

    def test_igmp_snooping_deployment(self, test='igmp_snooping_deployment'):
        """Check that igmp snooping bonding is properly configure

        mcast_snooping_enable and mcast-snooping-disable-flood-unregistered
        configured in br-int
        """
        LOG.info('Starting {} test.'.format(test))

        hypervisors = self._get_hypervisor_ip_from_undercloud(
            shell=CONF.nfv_plugin_options.undercloud_rc_file)

        result = []
        cmd = 'sudo ovs-vsctl --format=json list bridge br-int'
        checks = {'mcast_snooping_enable': True,
                  'mcast-snooping-disable-flood-unregistered': 'true'}

        for hypervisor_ip in hypervisors:
            output = shell_utils.run_command_over_ssh(hypervisor_ip, cmd)
            # ovs command returns boolean in small letters
            ovs_data = json.loads(output)
            ovs_data_filt = {}
            try:
                ovs_data_filt['mcast_snooping_enable'] = \
                    (ovs_data['data'][0]
                     [ovs_data['headings'].index('mcast_snooping_enable')])
                ovs_data_filt['mcast-snooping-disable-flood-unregistered'] = \
                    (dict(ovs_data['data'][0]
                          [ovs_data['headings'].index('other_config')][1])
                     ['mcast-snooping-disable-flood-unregistered'])
            except Exception:
                pass

            diff_checks_cmd = (set(checks.keys())
                               - set(ovs_data_filt.keys()))
            if len(diff_checks_cmd) > 0:
                result.append("{}. Missing checks: {}. Check ovs cmd output"
                              .format(hypervisor_ip,
                                      ', '.join(diff_checks_cmd)))

            for check in checks:
                if check not in diff_checks_cmd:
                    if ovs_data_filt[check] != checks[check]:
                        msg = ("{}. Check failed: {}. Expected: {} - Found: {}"
                               .format(hypervisor_ip, check, checks[check],
                                       ovs_data_filt[check]))
                        result.append(msg)

        self.assertTrue(len(result) == 0, '. '.join(result))
        return True

    def test_igmp_snooping(self, test='igmp_snooping'):
        """Test igmp snooping

           Having 2 hypervisors and 3 vms in each hypervisor. We configure a vm
           in each hypervisor sending traffic to a different group. We
           configure a different message for each group and different number of
           packets to be sent.
           Then we configure 2 vms in each hypervisor subscribed to each group.
           It is checked the following:
           - vms are properly subscribed
           - traffic in each interface in br-int
           - messages received in each vm and number of packets received
        """
        LOG.info('Starting {} test.'.format(test))

        if self.external_resources_data is None:
            raise ValueError('External resource data is required for the test')

        igmp_dict = {}
        test_setup_dict = self.test_setup_dict[test]
        if 'config_dict' in test_setup_dict and \
           'igmp_config' in test_setup_dict['config_dict']:
            igmp_dict = test_setup_dict['config_dict']['igmp_config'][0]

        if 'mcast_groups' in igmp_dict.keys() and \
           len(igmp_dict['mcast_groups']) == 2:
            pass
        else:
            raise ValueError('The test requires 2 multicast groups in '
                             'configuration file.')

        servers, key_pair = self.create_and_verify_resources(test=test)
        hypervisors = dict([[server['hypervisor_ip'], []]
                           for server in servers])
        for index, server in enumerate(servers):
            hypervisors[server['hypervisor_ip']].append(index)

        if [len(hypervisors[hyp]) for hyp in hypervisors.keys()] != [3, 3]:
            raise ValueError('The test requires 2 hypervisors and 3 vms in '
                             'each hypervisor.')

        for group in igmp_dict['mcast_groups']:
            group['rx_pkts'] = 0  # to be calculated
            group['msg'] = ''.join(random.choice(string.ascii_lowercase)
                                   for i in range(group['pkt_size']))
        # servers to be used
        hyp_0 = list(hypervisors.items())[0][1]
        hyp_1 = list(hypervisors.items())[1][1]
        servers[hyp_0[0]]['mcast'] = [{'role': 'traffic-runner', 'group': 0}]
        servers[hyp_0[1]]['mcast'] = [{'role': 'listener', 'group': 0}]
        servers[hyp_0[2]]['mcast'] = [{'role': 'listener', 'group': 1}]
        servers[hyp_1[0]]['mcast'] = [{'role': 'traffic-runner', 'group': 1}]
        servers[hyp_1[1]]['mcast'] = [{'role': 'listener', 'group': 0}]
        servers[hyp_1[2]]['mcast'] = [{'role': 'listener', 'group': 1}]

        errors = self.test_multicast_functionality(servers,
                                                   key_pair,
                                                   igmp_dict['mcast_groups'],
                                                   igmp_dict['pkts_tolerance'])

        self.assertTrue(len(errors) == 0, '. '.join(errors))
        LOG.info('Listeners received multicast traffic')

    def test_igmp_restart_ovs(self, test='igmp_restart_ovs'):
        """Test restart ovs

        Check that multicast configuration is not lost after ovs restart.
        Restart ovs and then execute test_igmp_snooping_deployment
        """
        LOG.info('Starting {} test.'.format(test))

        hypervisor_ips = self._get_hypervisor_ip_from_undercloud(
            shell=CONF.nfv_plugin_options.undercloud_rc_file)
        cmd = 'sudo systemctl restart openvswitch.service'
        for hyp in hypervisor_ips:
            shell_utils.run_command_over_ssh(hyp, cmd)
        self.test_igmp_snooping_deployment()

    def test_multicast_functionality(self, servers, key_pair, mcast_groups,
                                     pkts_tolerance):
        """common code to test most of the igmp snooping scenarios

        Functionality:
        - create multicast traffic runners
        - create multicast listeners
        - check multicast group creation
        - calculate traffic in ech interface
        - check received traffic in each listener
        param servers: list of servers
        param key_pair: key pair to connect to servers
        param mcast_groups: multicast groups to use
        param pkts_tolerance: tolerance for counting pkts in interfaces
        return a list with errors, or empty list if everything is ok
        """

        errors = []

        # get the ports name used for sending/reciving multicast traffic
        # it will be a different port than the management one that will be
        # connected to a switch in which igmp snooping is configured
        port_list = self.get_ovs_port_names(servers)

        # populate data
        hypervisors = {}
        for server in servers:
            server['ssh_source'] = self.get_remote_client(
                server['fip'],
                username=self.instance_user,
                private_key=key_pair['private_key'])
            hyper_ip = server['hypervisor_ip']
            if hyper_ip not in hypervisors.keys():
                hyp = []
                for group in mcast_groups:
                    hyp.append({'listeners': 0, 'traffic-runners': 0})
                hypervisors[hyper_ip] = hyp
            for idx, role in enumerate(server['mcast']):
                role['mcast_output'] = '/tmp/output-{}'.format(idx)
                group = role['group']
                if role['role'] == 'traffic-runner':
                    hypervisors[hyper_ip][group]['traffic-runners'] += 1
                    mcast_groups[group]['rx_pkts'] += (
                        mcast_groups[group]['tx_pkts'])
                elif role['role'] == 'listener':
                    hypervisors[hyper_ip][group]['listeners'] += 1
                LOG.info('Server {}: role {}, group: {}:{}, msg: {}'
                         .format(server['fip'],
                                 role['role'],
                                 mcast_groups[group]['ip'],
                                 mcast_groups[group]['port'],
                                 mcast_groups[group]['msg']))

        # calculate traffic in each interface
        for server in servers:
            server['tx_pkts'] = 0
            server['rx_pkts'] = 0
            for role in server['mcast']:
                if role['role'] == 'traffic-runner':
                    group = role['group']
                    server['tx_pkts'] += mcast_groups[group]['tx_pkts']
                elif role['role'] == 'listener':
                    group = role['group']
                    server['rx_pkts'] += mcast_groups[group]['rx_pkts']

        # kill multicast process if it exists
        cmd = "pids=$(ps -e -o \"cmd:50 \" -o \"|%p\" | " \
              "awk -F '|' '$1 ~ /multicast/ { print $2}');" \
              "if [[ ! -z $pids ]];then sudo kill -9 $pids;fi;"
        for server in servers:
            server['ssh_source'].exec_command(cmd)

        # start listeners
        for server in servers:
            for role in server['mcast']:
                if role['role'] == 'listener':
                    group = role['group']
                    cmd = 'sudo python /usr/local/bin/multicast_traffic.py ' \
                          '-r -g {0} -p {1} -c {2} > {3} 2>&1 &' \
                          .format(mcast_groups[group]['ip'],
                                  mcast_groups[group]['port'],
                                  mcast_groups[group]['tx_pkts'],
                                  role['mcast_output'])
                    server['ssh_source'].exec_command(cmd)

        # check groups are created
        for hyp in hypervisors.keys():
            groups = self.get_ovs_multicast_groups("br-int", hypervisor=hyp)
            for idx, group_info in enumerate(hypervisors[hyp]):
                num_listeners = len([grp for grp in groups if
                                    grp['GROUP'] == mcast_groups[idx]['ip']])
                if num_listeners != hypervisors[hyp][idx]['listeners']:
                    errors.append("Multicast groups not created properly: "
                                  "{} {}. ".format(mcast_groups[idx]['ip'],
                                                   mcast_groups[idx]['port']))

        # start servers
        stats_beg = {}
        stats_end = {}
        for hyp in hypervisors.keys():
            stats_beg[hyp] = self.get_ovs_interface_statistics(port_list[hyp],
                                                               hypervisor=hyp)
        for server in servers:
            for role in server['mcast']:
                if role['role'] == 'traffic-runner':
                    group = role['group']
                    cmd = 'sudo python /usr/local/bin/multicast_traffic.py ' \
                          '-s -g {0} -p {1} -m {2} -c {3} > {4} 2>&1 &' \
                          .format(mcast_groups[group]['ip'],
                                  mcast_groups[group]['port'],
                                  mcast_groups[group]['msg'],
                                  mcast_groups[group]['tx_pkts'],
                                  role['mcast_output'])
                    server['ssh_source'].exec_command(cmd)

        for hyp in hypervisors.keys():
            stats_end[hyp] = self.get_ovs_interface_statistics(port_list[hyp],
                                                               stats_beg[hyp],
                                                               hypervisor=hyp)

        # check groups has been removed
        for hyp in hypervisors.keys():
            groups = self.get_ovs_multicast_groups("br-int", hypervisor=hyp)
            for idx, group_info in enumerate(hypervisors[hyp]):
                num_listeners = len([grp for grp in groups if
                                    grp['GROUP'] == mcast_groups[idx]['ip']])
                if num_listeners != 0:
                    errors.append("Multicast groups not released properly: "
                                  "{} {}. ".format(mcast_groups[idx]['ip'],
                                                   mcast_groups[idx]['port']))

        # check traffic in listener and traffic runner interfaces
        for server in servers:
            stats = stats_end[server['hypervisor_ip']]
            tx_pkts_mcast = stats[server['other_port']]['tx_packets']
            rx_pkts_mcast = stats[server['other_port']]['rx_packets']
            tx_pkts_mgmt = stats[server['mgmt_port']]['tx_packets']
            rx_pkts_mgmt = stats[server['mgmt_port']]['rx_packets']

            LOG.info('{} Multicast Traffic stats, tx_pkts: {}, rx_pkts {}'
                     .format(server['fip'], tx_pkts_mcast, rx_pkts_mcast))
            LOG.info('{} Mgmt Traffic stats, tx_pkts: {}, rx_pkts {}.'
                     .format(server['fip'], tx_pkts_mgmt, rx_pkts_mgmt))
            if not (rx_pkts_mcast >= server['tx_pkts']
                    and rx_pkts_mcast <= (server['tx_pkts'] + pkts_tolerance)):
                errors.append("No traffic in traffic runner {}: {}. ".format(
                    server['fip'], rx_pkts_mcast))
            if not (tx_pkts_mcast >= server['rx_pkts']
                    and tx_pkts_mcast <= server['rx_pkts'] + pkts_tolerance):
                errors.append("No traffic in listener {}: {}. ".format(
                    server['fip'], tx_pkts_mcast))

        # check that messages were received properly
        for server in servers:
            for role in server['mcast']:
                if role['role'] == 'listener':
                    group = role['group']
                    get_mcast_results = ('cat {} | sort | uniq -c'
                                         .format(role['mcast_output']))
                    LOG.info('Reading results from {} instance.'
                             .format(server['fip']))
                    output = server['ssh_source'].exec_command(
                        get_mcast_results)
                    results = output.rstrip('\n').lstrip()
                    fail_results_msg = '{} unable to receive multicast ' \
                                       'traffic: {} '.format(server['fip'],
                                                             results)
                    if results != (str(mcast_groups[group]['rx_pkts'])
                                   + " " + mcast_groups[group]['msg']):
                        errors.append(fail_results_msg)
                    else:
                        LOG.info('{} received multicast traffic.'
                                 .format(server['fip']))

        return errors



# Copyright 2018 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import re
import six
import time


from nfv_tempest_plugin.tests.common import shell_utilities as shell_utils
from nfv_tempest_plugin.tests.scenario import base_test
from nfv_tempest_plugin.tests.scenario.qos_manager import QoSManagerMixin
from oslo_log import log as logging
from tempest import config

CONF = config.CONF
LOG = logging.getLogger('{} [-] nfv_plugin_test'.format(__name__))


class TestSriovScenarios(base_test.BaseTest, QoSManagerMixin):
    def __init__(self, *args, **kwargs):
        super(TestSriovScenarios, self).__init__(*args, **kwargs)

    def setUp(self):
        """Set up a single tenant with an accessible server

        If multi-host is enabled, save created server uuids.
        """
        super(TestSriovScenarios, self).setUp()
        # pre setup creations and checks read from

    def test_sriov_trusted_vfs(self, test='trustedvfs'):
        """Verify trusted virtual functions

        The test search 'trust on' configuration in the instance interfaces.
        """
        trusted_vfs_mac_addresses = []
        servers, key_pair = self.create_and_verify_resources(test=test)
        LOG.info('List ports and search for "trusted" in binding profile.')
        ports = self.ports_client.list_ports(device_id=servers[0]['id'])
        for port in ports['ports']:
            if 'trusted' in port['binding:profile'] and \
                    port['binding:profile']['trusted']:
                trusted_vfs_mac_addresses.append(port['mac_address'])
        self.assertNotEmpty(trusted_vfs_mac_addresses,
                            "No trusted VFs are attached to server")
        LOG.info('Test the "trust on" interface on the hypervisor.')
        result = shell_utils.\
            get_interfaces_from_overcloud_node(servers[0]['hypervisor_ip'])
        for mac_address in trusted_vfs_mac_addresses:
            re_string = r'.*{}.*'.format(mac_address)
            line = re.search(re_string, result)
            # Failed to locate MAC address on hypervisor
            if not line:
                raise ValueError("Failed to locate interface with MAC "
                                 "'{}' on hypervisor".format(mac_address))
            line = line.group(0)
            if mac_address in line:
                self.assertIn('trust on', line)
        LOG.info('The {} test passed.'.format(test))
        return True

    def test_sriov_double_tagging(self, test='double_tagging'):
        """Test SRIOV double tagging functionality

        The test require resource creator to setup initial test resources.
        Refer to the documentation regarding the test configuration.
        Note! - Both of the vlans should be allowed on the switch ports.
        """
        if self.external_resources_data is None:
            raise ValueError('External resource data is required for the test')

        LOG.info('Start SRIOV double tagging test.')
        servers, key_pair = self.create_and_verify_resources(test=test)
        if len(servers) != 4:
            raise ValueError('The test requires 4 instances.')

        if 'iface_vlan' and 'test_vlan' in \
                self.test_setup_dict[test]['vlan_config']:
            iface_vlan = \
                self.test_setup_dict[test]['vlan_config']['iface_vlan']
            test_vlan = self.test_setup_dict[test]['vlan_config']['test_vlan']
            LOG.info('Set test vlans {} and {} for the test'.format(iface_vlan,
                                                                    test_vlan))
        else:
            raise ValueError('The "iface_vlan" and "test_vlan" is missing from'
                             'the test config. Refer to the documentation. In '
                             'addition, make sure the "iface_vlan" and "test_v'
                             'lan" allowed in the switch to continue the test')

        trigger = None
        # Base vlan is the deployment vlan (neutron) and required by PF iface.
        base_vlan = ''
        for num, srv in enumerate(servers):
            srv_ports = self.ports_client.list_ports(
                device_id=srv['id'])['ports']
            for port in srv_ports:
                if port['binding:vnic_type'] in {'direct', 'direct-physical'}:
                    srv['test_port_type'] = port['binding:vnic_type']
                    srv['test_mac_addr'] = port['mac_address']
                    srv['test_ip_addr'] = '10.10.10.1{}'.format(num)

            if srv['test_port_type'] == 'direct-physical':
                srv['test_name'] = 'pf1'
            else:
                srv['test_name'] = 'vf{}'.format(num + 1)

            if trigger is None:
                cmd = "sudo ip link | awk -F ',' '/{}/ {{print $2}}' " \
                      "| tr -dc '0-9'".format(srv['test_mac_addr'])
                base_vlan = shell_utils.run_command_over_ssh(
                    srv['hypervisor_ip'], cmd)
                trigger = 'stop'

            vf_config = 'sudo python {script_path}/packages_deploy.py ' \
                        '--pip-packages scapy; sudo python ' \
                        '{script_path}/post_net_bootstrap.py --add-iface ' \
                        '--port-type {port_type} --mac {mac} --vlan ' \
                        '{iface_vlan} --addr {addr}/24'
            pf_config = vf_config + ' --base-vlan {base_vlan}'
            vf_config = vf_config.format(script_path=self.nfv_scripts_path,
                                         port_type='vf',
                                         mac=srv['test_mac_addr'],
                                         addr=srv['test_ip_addr'],
                                         iface_vlan=iface_vlan)
            pf_config = pf_config.format(script_path=self.nfv_scripts_path,
                                         port_type='pf',
                                         mac=srv['test_mac_addr'],
                                         addr=srv['test_ip_addr'],
                                         iface_vlan=iface_vlan,
                                         base_vlan=base_vlan)
            LOG.info('Create and configure virtual interface for '
                     '{}'.format(srv['test_name']))
            ssh_source = self.get_remote_client(srv['fip'],
                                                username=self.instance_user,
                                                private_key=key_pair[
                                                    'private_key'])
            output = ssh_source.exec_command(pf_config if
                                             srv['test_port_type']
                                             == 'direct-physical'
                                             else vf_config)
            srv['test_nic'] = output.split('\n')[-4].split(' ')[-1]

        vf1 = servers[0]

        scapy_sniff = 'sudo python {script_path}/scapy_traffic.py --sniff ' \
                      '--keep-sniff -i {iface} -c 5 > /dev/null 2>&1 &'
        scapy_icmp = 'sudo python {script_path}/scapy_traffic.py ' \
                     '--{pkt_type} -i {iface} -c 5 --src-mac {src_mac} ' \
                     '--src-ip {src_ip} --dst-mac {dst_mac} --dst-ip ' \
                     '{dst_ip} --iface-vlan {iface_vlan} --test-vlan ' \
                     '{test_vlan}'
        scapy_mpls = scapy_icmp + ' --raw-msg {raw_msg}'

        send_traffic_cmd = ''
        LOG.info('Start to sniff for ICMP traffic')
        for sniffer in servers:
            if sniffer['test_name'] in ['vf2', 'vf3', 'pf1']:
                ssh_source = self.get_remote_client(
                    sniffer['fip'], username=self.instance_user,
                    private_key=key_pair['private_key'])
                LOG.info('Start sniffing on {}'.format(sniffer['test_name']))
                ssh_source.exec_command(scapy_sniff.format(
                    iface=sniffer['test_nic']))
                s1 = scapy_icmp.format(script_path=self.nfv_scripts_path,
                                       pkt_type='icmp', iface=vf1['test_nic'],
                                       src_mac=vf1['test_mac_addr'],
                                       src_ip=vf1['test_ip_addr'],
                                       dst_mac=sniffer['test_mac_addr'],
                                       dst_ip=sniffer['test_ip_addr'],
                                       iface_vlan=iface_vlan,
                                       test_vlan=test_vlan)
                s2 = scapy_mpls.format(script_path=self.nfv_scripts_path,
                                       pkt_type='mpls', iface=vf1['test_nic'],
                                       src_mac=vf1['test_mac_addr'],
                                       src_ip=vf1['test_ip_addr'],
                                       dst_mac=sniffer['test_mac_addr'],
                                       dst_ip=sniffer['test_ip_addr'],
                                       iface_vlan=iface_vlan,
                                       test_vlan=test_vlan, raw_msg='testtest')
                send_traffic_cmd += '{}; {}; '.format(s1, s2)
        LOG.info('Send ICMP traffic')
        ssh_source = self.get_remote_client(vf1['fip'],
                                            username=self.instance_user,
                                            private_key=key_pair[
                                                'private_key'])
        LOG.info('Set scapy packets\n{}'.format(send_traffic_cmd))
        ssh_source.exec_command(send_traffic_cmd)

        LOG.info('Check for the results')
        test_results = []
        for sniffer in servers:
            if sniffer['test_name'] in ['vf2', 'vf3', 'pf1']:
                cmd = 'grep output /tmp/scapy_traffic.log | grep -o -e ' \
                      '"{[^}]*}" | tail -n2 || true'
                ssh_source = self.get_remote_client(
                    sniffer['fip'], username=self.instance_user,
                    private_key=key_pair['private_key'])
                output = ssh_source.exec_command(cmd)
                if output:
                    output = output.splitlines()
                    for option in output:
                        opt = eval(option)
                        if opt['pkt_type'] not in ['icmp', 'mpls']:
                            test_results.append(
                                '{} did not catch ICMP or MPLS '
                                'traffic'.format(sniffer['test_name']))
                        if opt['pkt_count'] != 5:
                            test_results.append(
                                '{} catch less packets than '
                                'expected'.format(sniffer['test_name']))
                else:
                    test_results.append('Output failed for '
                                        '{}'.format(sniffer['test_name']))

        for srv in servers:
            vf_remove = 'sudo python {script_path}/post_net_bootstrap.py ' \
                        '--del-iface --port-type vf --mac {mac} --vlan ' \
                        '{iface_vlan}'\
                .format(script_path=self.nfv_scripts_path,
                        mac=srv['test_mac_addr'], iface_vlan=iface_vlan)
            pf_remove = 'sudo python {script_path}/post_net_bootstrap.py ' \
                        '--del-iface --port-type pf --mac {mac} --base-vlan ' \
                        '{base_vlan}'.format(script_path=self.nfv_scripts_path,
                                             mac=srv['test_mac_addr'],
                                             base_vlan=base_vlan)
            remove_log_file = '; sudo rm -f /tmp/scapy_traffic.log'
            vf_remove += remove_log_file
            pf_remove += remove_log_file
            ssh_source = self.get_remote_client(srv['fip'],
                                                username=self.instance_user,
                                                private_key=key_pair[
                                                    'private_key'])
            ssh_source.exec_command(pf_remove if
                                    srv['test_port_type']
                                    == 'direct-physical' else vf_remove)

        self.assertEmpty(test_results, test_results)

    def test_guests_with_min_bw(self, test='sriov_min_bw_qos'):
        """Test SR-IOV minimum bandwith (Nova)

        Spawn a guest with a minimum already applied to port
        """
        # Create servers
        qos_rules = {}
        for rule in self.test_setup_dict[test]['qos_rules']:
            qos_rules.update(rule)
        self.test_setup_dict[test]
        self.create_qos_policy_with_rules(**qos_rules)
        resource_args = {'set_qos': True}
        servers, key_pair = self.create_and_verify_resources(test=test,
                                                             **resource_args)
        # Iterate over servers
        for server in servers:
            self.check_qos_attached_to_guest(server,
                                             min_bw=True)

    def test_guests_set_min_qos(self, test='sriov_min_bw_qos'):
        """Test SR-IOV minimum QoS

        Spawn a guest and set (neutron) minimum QoS to port already up
        """
        qos_rules = {}
        for rule in self.test_setup_dict[test]['qos_rules']:
            qos_rules.update(rule)
        self.create_qos_policy_with_rules(**qos_rules)
        # Create servers
        resource_args = {'set_qos': False}
        servers, key_pair = self.create_and_verify_resources(test=test,
                                                             **resource_args)
        # search for min_qos port
        min_qos_port = ''
        for port in self.os_admin.ports_client.list_ports(
                device_id=self.servers[0]['id'])['ports']:
            if 'min-qos' in port['name']:
                min_qos_port = port['id']
        # Update QoS of the port
        self.update_port(min_qos_port,
                         **{'qos_policy_id': self.qos_policy_groups['id']})
        # Wait the qos_policy update
        msg = "qos policy was not added to port {}".join(min_qos_port)
        time.sleep(5)
        client = self.os_admin.ports_client
        self.assertIsNotNone(
            client.show_port(min_qos_port)['port']['qos_policy_id'],
            msg)
        for server in servers:
            self.check_qos_attached_to_guest(server,
                                             min_bw=True)

    def test_sriov_free_resource(self, test='sriov_reset_resources'):
        """Test_sriov_free_resources

        The method checks if sriov nics are released after guest/port deletion.
        Verification is run before test starts and at the end
        """
        # Check resources are free in computes
        self.test_setup_dict['sriov_reset_resources'] = \
            {'flavor-id': self.flavor_ref, 'router': True, 'aggregate': None}
        default_port_type = {'ports_filter': 'external,direct'}
        resource_args = {'num_servers': 4,
                         'srv_details': {0: default_port_type,
                                         1: default_port_type,
                                         2: default_port_type,
                                         3: default_port_type}}
        # Set test parameters
        kw_args = {}
        osp_release = self.get_osp_release()
        # Starting from OSP13, installation is containerized
        kw_args['command'] = "sudo ip link show"
        kw_args['file_path'] = \
            '/var/lib/config-data/nova_libvirt/etc/nova/nova.conf'
        if osp_release < 13:
            kw_args['file_path'] = '/etc/nova/nova.conf'
        kw_args['search_param'] = \
            {'section': 'pci', 'value': 'passthrough_whitelist'}
        """ Regexp search VF interfaces with neutron MAC prefix,
        [Cr] is for catching the following options: MAC|link/ether for
        different OS versions """
        kw_args['filter_regexp'] = \
            r"\s+vf.*[Cr]\sfa:16:3e:[a-fA-F0-9:]{2}.*"
        kw_args['servers_ips'] = self.\
            _get_hypervisor_ip_from_undercloud(
            shell=CONF.nfv_plugin_options.undercloud_rc_file)
        kw_args['multi_key_values'] = True
        # Verify empty machines are reset back from previous tests
        result = shell_utils. \
            run_hypervisor_command_build_from_config(**kw_args)
        # Iterate regexp result if found some thing fail test
        msg = "Parameters not in [] required state \n {}".format(result)
        for ihost in result.keys():
            assert len(result[ihost]) == 0, msg.replace("[]", ihost)
        # Create resources:
        self.create_and_verify_resources(test=test, **resource_args)
        # Check resources are free in computes
        for server in self.servers:
            ports_list = (
                self.os_admin.ports_client.list_ports(
                    device_id=server['id']))['ports']
            delete_ports = [x['id'] for x in ports_list]
            # Delete computes and ports
            self.os_primary.servers_client.delete_server(server['id'])
            for port in delete_ports:
                self.os_admin.ports_client.delete_port(port)
        # Verify empty machines are reset back after tests
        result = shell_utils. \
            run_hypervisor_command_build_from_config(**kw_args)
        # Iterate regexp result if found some thing fail test
        for ihost in result.keys():
            assert len(result[ihost]) == 0, msg.replace("[]", ihost)

    def test_sriov_max_qos(self, test='max_qos'):
        """Test SRIOV MAX QoS functionality

        The test require [nfv_plugin_options ]
        use_neutron_api_v2 = true in tempest.config.
        Test also requires QoS neutron settings.
        The test deploy 3 vms. one iperf server receive traffic from
        two iperf clients, with max_qos defined run against iperf server.
        The test search for Traffic per second and compare against ports
        seeings
        """

        LOG.info('Start SRIOV Max QoS test.')
        kwargs = {}
        qos_rules = \
            json.loads(CONF.nfv_plugin_options.max_qos_rules)
        qos_rules_list = [x for x in qos_rules]
        servers, key_pair = self.create_and_verify_resources(
            test=test, num_servers=3, **kwargs)
        if len(servers) != 3:
            raise ValueError('The test requires 3 instances.')
        # Max QoS configuration to server ports
        LOG.info('Create QoS Policies...')
        qos_policies = [self.create_qos_policy_with_rules(
            use_default=False, **i) for i in qos_rules_list]
        self.run_iperf_test(qos_policies, servers, key_pair)
        self.collect_iperf_results(qos_rules_list, servers, key_pair)

    def test_sriov_min_qos(self, test='min_qos'):
        """Test SRIOV MIN QoS functionality

        SUPPORTED: Mellanox NICS only
        The test require [nfv_plugin_options ]
        use_neutron_api_v2 = true in tempest.config.
        Test also requires QoS neutron settings.
        The test deploy 3 vms. one iperf server receive traffic from
        two iperf clients, with min_qos defined run against iperf server.
        The test search for Traffic per second and compare against ports
        seeings
        """

        LOG.info('Start SRIOV Min QoS test, search for Mellanox nics')
        # Check setup contains Mellanox nics.
        kw_args = dict()
        kw_args['command'] = r"sudo lshw -class network -short | grep "
        kw_args['file_path'] = \
            '/var/lib/config-data/nova_libvirt/etc/nova/nova.conf'
        kw_args['search_param'] = \
            {'section': 'pci', 'value': 'passthrough_whitelist'}
        """ Regexp search Mellanox connect-x """
        kw_args['filter_regexp'] = \
            r".*\[ConnectX\-5\].*"
        kw_args['servers_ips'] = self. \
            _get_hypervisor_ip_from_undercloud(
            shell=CONF.nfv_plugin_options.undercloud_rc_file)
        kw_args['multi_key_values'] = True
        LOG.info('Start SRIOV Min QoS test.')
        result = shell_utils. \
            run_hypervisor_command_build_from_config(**kw_args)
        msg = "no nics supporting sriov min bw"
        self.assertTrue(
            len(result) > 0, msg)
        qos_rules = \
            json.loads(CONF.nfv_plugin_options.min_qos_rules)
        qos_rules_list = [x for x in qos_rules]
        LOG.info('SRIOV Min QoS test, '
                 'learn provider network attached to device.')
        # Assuming both computes have the same hw
        # searching for physnet of Mellanox
        devices = \
            [re.split(r'\s+', i)[1] for i in six.next(six.itervalues(result))]
        pci_list = shell_utils.\
            get_value_from_ini_config(six.next(six.iterkeys(result)),
                                      kw_args['file_path'],
                                      kw_args['search_param']['section'],
                                      kw_args['search_param']['value'],
                                      kw_args['multi_key_values'])
        pci_list = '[' + pci_list + ']'
        json_list = json.loads(pci_list.replace("\n", ","))
        net_name = [pci_item['physical_network'] for pci_item in json_list
                    if pci_item['devname'] == devices[0]]

        LOG.info('SRIOV Min QoS test, create test vms.')
        # Test deploy 3 VMS on singlr hypervisor.
        hyper = self.hypervisor_client.list_hypervisors()['hypervisors']

        kw_test = \
            {'availability_zone': {'hyper_hosts': [hyper[0]
                                                   ['hypervisor_hostname']]}}
        default_port_type = \
            {'ports_filter': "{}:{}".format('external,direct', net_name[0])}
        kw_test['num_servers'] = 3
        kw_test['srv_details'] = {0: default_port_type,
                                  1: default_port_type,
                                  2: default_port_type}
        servers, key_pair = self.create_and_verify_resources(
            test=test, **kw_test)
        if len(servers) != 3:
            raise ValueError('The test requires 3 instances.')
        # Min QoS configuration to server ports
        LOG.info('Create QoS Policies...')
        qos_policies = [self.create_qos_policy_with_rules(
            use_default=True, **i) for i in qos_rules_list]
        LOG.info('SRIOV Min QoS test, run test.')
        self.run_iperf_test(qos_policies, servers, key_pair)
        LOG.info('SRIOV Min QoS test, check test result.')
        self.collect_iperf_results(qos_rules_list, servers, key_pair)



# Copyright 2019 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import math
import yaml

from nfv_tempest_plugin.tests.common import shell_utilities as shell_utils
from nfv_tempest_plugin.tests.scenario import base_test
from oslo_log import log as logging
from tempest import config

CONF = config.CONF
LOG = logging.getLogger('{} [-] nfv_plugin_test'.format(__name__))


class TestDeriveParamsScenarios(base_test.BaseTest):
    def __init__(self, *args, **kwargs):
        super(TestDeriveParamsScenarios, self).__init__(*args, **kwargs)

    def setUp(self):
        """Set up a single tenant with an accessible server

        If multi-host is enabled, save created server uuids.
        """
        super(TestDeriveParamsScenarios, self).setUp()

    # Gets the numa nodes list
    def _get_numa_nodes(self, hypervisor_ip):
        nodes = []
        cmd = "sudo lscpu -p=NODE | grep -v ^#"
        output = shell_utils.run_command_over_ssh(hypervisor_ip, cmd)
        for line in output.split('\n'):
            if line:
                node = int(line.strip(' '))
                if node not in nodes:
                    nodes.append(node)
        return nodes

    def _get_nodes_cores_info(self, hypervisor_ip):
        dict_cpus = {}
        cmd = "sudo lscpu -p=NODE,CORE,CPU | grep -v ^#"
        output = shell_utils.run_command_over_ssh(hypervisor_ip, cmd)
        for line in output.split('\n'):
            if line:
                cpu_info = line.split(',')
                node = int(cpu_info[0])
                cpu = int(cpu_info[1])
                thread = int(cpu_info[2])
                # CPU and NUMA node together forms a unique value, as cpu is
                # specific to a NUMA node
                # NUMA node id and cpu id tuple is used for unique key
                dict_key = node, cpu
                if dict_key in dict_cpus:
                    if thread not in dict_cpus[dict_key]['thread_siblings']:
                        dict_cpus[dict_key]['thread_siblings'].append(thread)
                else:
                    cpu_item = {}
                    cpu_item['thread_siblings'] = [thread]
                    cpu_item['cpu'] = cpu
                    cpu_item['numa_node'] = node
                    dict_cpus[dict_key] = cpu_item
        return dict_cpus

    # Gets the DPDK NIC's mapping with NIC physical name and driver info
    # for the given MAC.
    def _get_dpdk_nics_mapping(self, hypervisor_ip, mac):
        cmd = "sudo cat /var/lib/os-net-config/dpdk_mapping.yaml"
        output = shell_utils.run_command_over_ssh(hypervisor_ip, cmd)
        dpdk_nics_map = yaml.safe_load(output)
        for dpdk_nic_map in dpdk_nics_map:
            if dpdk_nic_map['mac_address'] == mac:
                return dpdk_nic_map
        else:
            msg = ("Unable to determine DPDK NIC Mapping for "
                   "MAC: '%(mac)s'" % {'mac': mac})
            raise Exception(msg)

    # Gets the DPDK NIC's NUMA info
    def _get_dpdk_nics_info(self, hypervisor_ip):
        dpdk_nics_info = []
        dpdk_nics = []
        cmd = ("sudo ovs-vsctl --columns=name,type,admin_state "
               "--format=json list interface")
        output = shell_utils.run_command_over_ssh(hypervisor_ip, cmd)
        nics = json.loads(output)
        for nic in nics.get('data', []):
            if nic and str(nic[1]) == 'dpdk' and str(nic[2]) == 'up':
                dpdk_nics.append(str(nic[0]))
        if dpdk_nics:
            cmd = ("sudo ovs-vsctl --column=mac-in-use,mtu,status "
                   "--format=json list interface " + ' '.join(dpdk_nics))
            output = shell_utils.run_command_over_ssh(hypervisor_ip, cmd)
            nics_info = json.loads(output)
            for nic_info in nics_info.get('data', []):
                data = {}
                data['mac'] = nic_info[0]
                data['mtu'] = nic_info[1]
                for field in nic_info[2][1]:
                    if field[0] == 'numa_id':
                        data['numa_node'] = int(field[1])
                        dpdk_nic_map = self._get_dpdk_nics_mapping(
                            hypervisor_ip, nic_info[0])
                        data['nic'] = dpdk_nic_map['name']
                        data['pci'] = dpdk_nic_map['pci_address']
                        dpdk_nics_info.append(data)
        return dpdk_nics_info

    # Gets the total physical memory.
    def _get_physical_memory(self, hypervisor_ip):
        mem_total_kb = 0
        cmd = "sudo dmidecode --type memory | grep 'Size' | grep '[0-9]'"
        output = shell_utils.run_command_over_ssh(hypervisor_ip, cmd)
        for line in output.split('\n'):
            if line:
                mem_info = line.split(':')[1].strip()
                mem_val = mem_info.split(' ')
                mem_unit = mem_val[1].strip(' ').lower()
                if mem_unit == 'kb':
                    memory_kb = int(mem_val[0].strip(' '))
                elif mem_unit == 'mb':
                    memory_kb = (int(mem_val[0].strip(' ')) * 1024)
                elif mem_unit == 'gb':
                    memory_kb = (int(mem_val[0].strip(' ')) * 1024 * 1024)
                mem_total_kb += memory_kb
        return (mem_total_kb / 1024)

    # Gets the DPDK PMD core list
    # Find the right logical CPUs to be allocated along with its
    # siblings for the PMD core list
    def _get_dpdk_core_list(self, hypervisor_ip, cpus, dpdk_nics_numa_info,
                            dpdk_nic_numa_cores_count=1):
        dpdk_core_list = []
        dpdk_nics_numa_nodes = [dpdk_nic['numa_node']
                                for dpdk_nic in dpdk_nics_numa_info]

        numa_cores = {}
        numa_nodes = self._get_numa_nodes(hypervisor_ip)
        for node in numa_nodes:
            if node in dpdk_nics_numa_nodes:
                numa_cores[node] = dpdk_nic_numa_cores_count
            else:
                numa_cores[node] = 1

        numa_nodes_threads = {}

        for cpu in cpus:
            if not cpu['numa_node'] in numa_nodes_threads:
                numa_nodes_threads[cpu['numa_node']] = []
            numa_nodes_threads[cpu['numa_node']].extend(cpu['thread_siblings'])

        for node, node_cores_count in numa_cores.items():
            numa_node_min = min(numa_nodes_threads[node])
            cores_count = node_cores_count
            for cpu in cpus:
                if cpu['numa_node'] == node:
                    # Adds threads from core which is not having least thread
                    if numa_node_min not in cpu['thread_siblings']:
                        dpdk_core_list.extend(cpu['thread_siblings'])
                        cores_count -= 1
                        if cores_count == 0:
                            break
        return ','.join([str(thread) for thread in sorted(dpdk_core_list)])

    # Gets host cpus
    def _get_host_cpus_list(self, hypervisor_ip, cpus):
        host_cpus_list = []

        numa_nodes_threads = {}
        # Creates a list for all available threads in each NUMA nodes
        for cpu in cpus:
            if not cpu['numa_node'] in numa_nodes_threads:
                numa_nodes_threads[cpu['numa_node']] = []
            numa_nodes_threads[cpu['numa_node']].extend(
                cpu['thread_siblings'])

        for numa_node in sorted(numa_nodes_threads.keys()):
            node = int(numa_node)
            # Gets least thread in NUMA node
            numa_node_min = min(numa_nodes_threads[numa_node])
            for cpu in cpus:
                if cpu['numa_node'] == node:
                    # Adds threads from core which is having least thread
                    if numa_node_min in cpu['thread_siblings']:
                        host_cpus_list.extend(cpu['thread_siblings'])
                        break

        return ','.join([str(thread) for thread in sorted(host_cpus_list)])

    # Computes round off MTU value in bytes
    # example: MTU value 9000 into 9216 bytes
    def _roundup_mtu_bytes(self, mtu):
        max_div_val = int(math.ceil(float(mtu) / float(1024)))
        return (max_div_val * 1024)

    # Calculates socket memory for a NUMA node
    def _calculate_node_socket_memory(self, numa_node, dpdk_nics_numa_info,
                                      overhead, packet_size_in_buffer,
                                      minimum_socket_memory):
        distinct_mtu_per_node = []
        socket_memory = 0

        # For DPDK numa node
        for nics_info in dpdk_nics_numa_info:
            if (numa_node == nics_info['numa_node']
                    and not nics_info['mtu'] in distinct_mtu_per_node):
                distinct_mtu_per_node.append(nics_info['mtu'])
                roundup_mtu = self._roundup_mtu_bytes(nics_info['mtu'])
                socket_memory += (((roundup_mtu + overhead)
                                   * packet_size_in_buffer) / (1024 * 1024))

        # For Non DPDK numa node
        if socket_memory == 0:
            socket_memory = minimum_socket_memory
        # For DPDK numa node
        else:
            socket_memory += 512

        socket_memory_in_gb = int(socket_memory / 1024)
        if socket_memory % 1024 > 0:
            socket_memory_in_gb += 1
        return (socket_memory_in_gb * 1024)

    # Gets the socket memory
    def _get_dpdk_socket_memory(self, hypervisor_ip, dpdk_nics_numa_info,
                                minimum_socket_memory=1500):
        dpdk_socket_memory_list = []
        overhead = 800
        packet_size_in_buffer = 4096 * 64
        numa_nodes = self._get_numa_nodes(hypervisor_ip)
        for node in numa_nodes:
            socket_mem = self._calculate_node_socket_memory(
                node, dpdk_nics_numa_info, overhead,
                packet_size_in_buffer,
                minimum_socket_memory)
            dpdk_socket_memory_list.append(socket_mem)
        return ','.join([str(sm) for sm in dpdk_socket_memory_list])

    # Gets nova cpus
    def _get_nova_cpus_list(self, hypervisor_ip, cpus, dpdk_cpus, host_cpus):
        nova_cpus_list = []
        threads = []
        # Creates a list for all available threads in each NUMA nodes
        for cpu in cpus:
            threads.extend(cpu['thread_siblings'])
        exclude_cpus_list = dpdk_cpus.split(',')
        exclude_cpus_list.extend(host_cpus.split(','))
        for thread in threads:
            if not str(thread) in exclude_cpus_list:
                nova_cpus_list.append(thread)
        return ','.join([str(thread) for thread in sorted(nova_cpus_list)])

    # Gets host isolated cpus
    def _get_host_isolated_cpus_list(self, dpdk_cpus, nova_cpus):
        host_isolated_cpus_list = [int(cpu) for cpu in dpdk_cpus.split(',')]
        host_isolated_cpus_list.extend(
            [int(cpu) for cpu in nova_cpus.split(',')])
        return (','.join([str(thread)
                for thread in sorted(host_isolated_cpus_list)]))

    # Gets the CPU model and flags
    def _get_cpu_details(self, hypervisor_ip):
        cmd = "sudo lscpu | grep 'Model name';sudo lscpu | grep 'Flags'"
        output = shell_utils.run_command_over_ssh(hypervisor_ip, cmd)
        if output:
            cpu_model = ""
            cpu_flags = []
            params = output.split('\n')
            if params:
                for param in params:
                    if "Model name" in param:
                        cpu_model = param.split(':')[1].strip(' \n')
                    elif "Flags" in param:
                        cpu_flags = param.split(':')[1].strip(' \n').split(' ')
            return cpu_model, cpu_flags
        else:
            msg = "Unable to determine 'CPU Model name'"
            raise Exception(msg)

    # Checks default 1GB hugepages support
    def _is_supported_default_hugepages(self, flags):
        return ('pdpe1gb' in flags)

    # Derives kernel_args parameter
    def _get_kernel_args(self, hypervisor_ip, hugepage_alloc_perc=50):
        kernel_args = {}
        cpu_model, cpu_flags = self._get_cpu_details(hypervisor_ip)
        if not self._is_supported_default_hugepages(cpu_flags):
            raise Exception("default huge page size 1GB is not supported")
        total_memory = self._get_physical_memory(hypervisor_ip)
        hugepages = int(float((total_memory / 1024) - 4)
                        * (hugepage_alloc_perc / float(100)))
        if cpu_model.startswith('Intel'):
            kernel_args['intel_iommu'] = 'on'
        kernel_args['iommu'] = 'pt'
        kernel_args['default_hugepagesz'] = '1GB'
        kernel_args['hugepagesz'] = '1G'
        kernel_args['hugepages'] = str(hugepages)
        return kernel_args

    # converts range list into number list
    # here input parameter and return value as list
    # example: ["12-14", "^13", "17"] into [12, 14, 17]
    def _convert_range_to_number_list(self, range_list):
        num_list = []
        exclude_num_list = []
        try:
            for val in range_list.split(','):
                val = val.strip(' ')
                if '^' in val:
                    exclude_num_list.append(int(val[1:]))
                elif '-' in val:
                    split_list = val.split("-")
                    range_min = int(split_list[0])
                    range_max = int(split_list[1])
                    num_list.extend(range(range_min, (range_max + 1)))
                else:
                    num_list.append(int(val))
        except ValueError as exc:
            err_msg = ("Invalid number in input param "
                       "'range_list': %s" % exc)
            raise Exception(err_msg)

        # here, num_list is a list of integers
        return (','.join([str(num)
                for num in num_list if num not in exclude_num_list]))

    def _get_node_nfv_status(self, hypervisor_ip):
        dpdk_status = False
        sriov_status = False
        cmd = ("sudo ovs-vswitchd --version | "
               "awk '{ if ($1 == \"DPDK\") print 1; }';"
               "echo '|';sudo cat /etc/puppet/hieradata/service_names.json")
        output = shell_utils.run_command_over_ssh(hypervisor_ip, cmd)
        if output:
            params = output.split('|')
            if params:
                if '1' in params[0].strip('\n'):
                    dpdk_status = True
                service_names = json.loads(params[1])
                if (service_names['service_names']
                        and "neutron_sriov_agent"
                        in service_names['service_names']):
                    sriov_status = True
        return dpdk_status, sriov_status

    # Derives the OvS DPDK and SRIOV parameters
    def _derive_parameters(self, hypervisor_ip,
                           dpdk_status, sriov_status):
        derived_params = {}
        dpdk_nics_info = self._get_dpdk_nics_info(hypervisor_ip)
        dict_cpus = self._get_nodes_cores_info(hypervisor_ip)
        cpus = list(dict_cpus.values())
        host_cpus_list = self._get_host_cpus_list(hypervisor_ip, cpus)
        dpdk_core_list = ""
        if dpdk_status:
            dpdk_core_list = self._get_dpdk_core_list(hypervisor_ip,
                                                      cpus, dpdk_nics_info)
            derived_params["OvsPmdCoreList"] = dpdk_core_list
            derived_params["OvsDpdkSocketMemory"] = \
                self._get_dpdk_socket_memory(hypervisor_ip, dpdk_nics_info)
            derived_params["OvsDpdkCoreList"] = host_cpus_list
        if dpdk_status or sriov_status:
            nova_cpus = self._get_nova_cpus_list(hypervisor_ip, cpus,
                                                 dpdk_core_list,
                                                 host_cpus_list)
            derived_params["NovaVcpuPinSet"] = nova_cpus
            derived_params["IsolCpusList"] = \
                self._get_host_isolated_cpus_list(dpdk_core_list, nova_cpus)
            derived_params["KernelArgs"] = \
                self._get_kernel_args(hypervisor_ip)
            # Recommended reserved host memory is 4096
            derived_params["NovaReservedHostMemory"] = '4096'
        return derived_params

    # Gets the OvS DPDK and SRIOV parameters current deployment values
    def _get_deployment_values(self, hypervisor_ip,
                               dpdk_status, sriov_status):
        host_params = {}
        retrive_host_params = {}
        if sriov_status:
            retrive_host_params = {
                'IsolCpusList': {'action': 'command',
                                 'cmd': (r"sudo cat /etc/tuned/bootcmdline | "
                                         r"grep -P -o 'nohz_full=.+?\s{1,}' |"
                                         r" sed 's/nohz_full=//'")},
                'KernelArgs': {'action': 'command',
                               'cmd': 'sudo cat /proc/cmdline'},
                'NovaReservedHostMemory': {'action': 'ini',
                                           'file_path': ('/var/lib/'
                                                         'config-data/'
                                                         'puppet-generated/'
                                                         'nova_libvirt/'
                                                         'etc/nova/'
                                                         'nova.conf'),
                                           'section': 'DEFAULT',
                                           'value': 'reserved_'
                                                    'host_memory_mb'},
                'NovaVcpuPinSet': {'action': 'ini',
                                   'file_path': ('/var/lib/config-data/'
                                                 'puppet-generated/'
                                                 'nova_libvirt/etc/nova/'
                                                 'nova.conf'),
                                   'section': 'DEFAULT',
                                   'value': 'vcpu_pin_set'}
            }
        if dpdk_status:
            retrive_host_params = {

                'IsolCpusList': {'action': 'command',
                                 'cmd': (r"sudo cat /etc/tuned/bootcmdline | "
                                         r"grep -P -o 'nohz_full=.+?\s{1,}' |"
                                         r" sed 's/nohz_full=//'")},
                'KernelArgs': {'action': 'command',
                               'cmd': 'sudo cat /proc/cmdline'},
                'NovaReservedHostMemory': {'action': 'ini',
                                           'file_path': ('/var/lib/'
                                                         'config-data/'
                                                         'puppet-generated/'
                                                         'nova_libvirt/etc/'
                                                         'nova/nova.conf'),
                                           'section': 'DEFAULT',
                                           'value': 'reserved_'
                                                    'host_memory_mb'},
                'NovaVcpuPinSet': {'action': 'ini',
                                   'file_path': ('/var/lib/config-data/'
                                                 'puppet-generated/'
                                                 'nova_libvirt/etc/nova/'
                                                 'nova.conf'),
                                   'section': 'DEFAULT',
                                   'value': 'vcpu_pin_set'},
                'OvsDpdkCoreList': {'action': 'command',
                                    'cmd': (r"sudo pgrep ovsdb-server | xargs "
                                            r"taskset -cp | grep -P -o '\d+' |"
                                            r" tail -n +2 | paste -s -d, -")},
                'OvsDpdkSocketMemory': {'action': 'command',
                                        'cmd': ("sudo ovs-vsctl get "
                                                "Open_vSwitch . other_config"
                                                ":dpdk-socket-mem")},
                'OvsPmdCoreList': {'action': 'command',
                                   'cmd': ("sudo ovs-appctl dpif-netdev/"
                                           "pmd-rxq-show | grep core_id | "
                                           "cut -d ' ' -f 6 | "
                                           "sed -e 's/://' | paste -s -d, -")},
            }
        for param in retrive_host_params:
            if retrive_host_params[param]['action'] == 'command':
                cmd = retrive_host_params[param]['cmd']
                result = shell_utils.run_command_over_ssh(hypervisor_ip, cmd)
            elif retrive_host_params[param]['action'] == 'ini':
                file_path = retrive_host_params[param]['file_path']
                section = retrive_host_params[param]['section']
                value = retrive_host_params[param]['value']
                result = shell_utils.\
                    get_value_from_ini_config(hypervisor_ip,
                                              file_path, section,
                                              value)
            host_params[param] = result.strip('\n').strip('"').strip()
            if (param in ['OvsDpdkCoreList', 'OvsPmdCoreList',
                          'NovaVcpuPinSet', 'IsolCpusList']):
                host_params[param] = \
                    self._convert_range_to_number_list(host_params[param])
            elif param == 'KernelArgs':
                kernel_args_str = host_params[param]
                kernel_args = kernel_args_str.split(' ')
                required_kernel_args = {}
                for arg in kernel_args:
                    kernel_param = arg.split('=')
                    if (('hugepages' in kernel_param[0])
                            or ('iommu' in kernel_param[0])):
                        required_kernel_args[str(kernel_param[0].strip())] = \
                            str(kernel_param[1].strip())
                host_params[param] = required_kernel_args
        return host_params

    def test_derived_parameters(self, test='derive_params'):
        """Test Derived Parameters

        This test compares the each parameters using derived values
        and current deployment values
        """
        derived_params = {}
        host_params = {}
        failures = []
        hypervisor_ip = self._get_hypervisor_ip_from_undercloud(
            shell=CONF.nfv_plugin_options.undercloud_rc_file)[0]

        dpdk_status, sriov_status = self._get_node_nfv_status(hypervisor_ip)
        derived_params = self._derive_parameters(hypervisor_ip,
                                                 dpdk_status,
                                                 sriov_status)
        LOG.info("Derived parameters: {}".format(derived_params))
        host_params = self._get_deployment_values(hypervisor_ip,
                                                  dpdk_status,
                                                  sriov_status)
        LOG.info("Deployment parameters: {}".format(host_params))
        for param in host_params:
            if param not in derived_params:
                error = ("Derived parameter '{}' won't be parsed from "
                         "current deployment").format(param)
                failures.append(error)
                continue
            if host_params[param] != derived_params[param]:
                error = ("Derived parameter '{p}' {d_p} is not equal to "
                         "{h_p}").format(p=param,
                                         d_p=derived_params[param],
                                         h_p=host_params[param])
                failures.append(error)
        if failures:
            raise Exception(failures)
        else:
            return True



#!/bin/env python
# Copyright 2019 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# The PostNetBootstrap script is aims to configure the post boot networking
# like virtual interface on the selected interface.

import argparse
import logging
import os
import subprocess
import sys

logging.basicConfig(filename='/var/log/messages', filemode='a',
                    format='%(asctime)s %(name)s %(levelname)s: %(message)s',
                    datefmt="%h %d %H:%M:%S", level=logging.INFO)
handler = logging.StreamHandler(sys.stdout)
logger = logging.getLogger("PostNetBootstrap")
logger.addHandler(handler)

parser = argparse.ArgumentParser(
    usage='''
    PostNetBootstrap test config script

    To add the VF based virtual interface and install scapy:
    $ python script.py --add-iface --port-type vf --mac aa:bb:cc:dd:ee:ff \
--vlan 10 --addr 10.10.10.10/24 --install-scapy

    To add the PF based virtual interface:
    $ python script.py --add-iface --port-type pf --mac aa:bb:cc:dd:ee:ff \
--vlan 10 --addr 10.10.10.12/24 --base-vlan 110

    To remove the VF based virtual interface:
    $ python script.py --del-iface --port-type vf --mac aa:bb:cc:dd:ee:ff \
--vlan 10

    To remove the PF based virtual interface:
    $ python script.py --del-iface --port-type pf --mac aa:bb:cc:dd:ee:ff
--base-vlan 110''')
iface = parser.add_mutually_exclusive_group()
iface.add_argument('--add-iface', help='Add virtual interface to server',
                   required=False, action='store_true')
iface.add_argument('--del-iface', help='Delete virtual interface from server',
                   required=False, action='store_true')
parser.add_argument('--port-type', help='The type of the virtual interface '
                                        'that should be created. VF or PF. '
                                        'The VF port, created as a regular '
                                        'virtual vlan interface. The PF port, '
                                        'is the direct passthrough to the '
                                        'instance, should be configured with '
                                        'the additional (base) vlan interface.'
                                        ' The test vlan will reside on top of'
                                        ' it',
                    choices=['vf', 'pf'], default='vf')
parser.add_argument('--mac', help='The mac address of the port that should '
                                  'be used for configuration')
parser.add_argument('--vlan', help='The vlan id that should be configured '
                                   'on the interface')
parser.add_argument('--addr', help='The address with the subnet that should '
                                   'be configured on the interface. '
                                   'Ex. 10.10.10.10/24')
parser.add_argument('--base-vlan', help='The vlan id that will be used as a '
                                        'based vlan for the PF. Look for type '
                                        'argument for the details.')
args = parser.parse_args()


def execute_shell_command(cmd):
    """Execute shell command

    The subprocess.check_output executes command provided as list.
    If the command will be provided as string, it will be converted to list
    and then executed.
    """
    if not isinstance(cmd, list):
        cmd = cmd.split()
    try:
        logger.info('Execute command: {}'.format(cmd))
        output = subprocess.check_output(cmd)
    except subprocess.CalledProcessError as e:
        logger.info('Command failed: {}'.format(e))
        raise
    return output


def check_existing_interfaces():
    """Check and locate existing network interfaces on instance"""
    ifaces = []
    nics = os.listdir('/sys/class/net/')
    nics.remove('lo')
    for nic in nics:
        nic_path = '/sys/class/net/{}/address'.format(nic)
        with open(nic_path, 'r') as f:
            mac = f.read().rstrip()
        ifaces.append({nic: mac})
    return ifaces


def choose_requested_interface(nics, mac):
    """The function will return an interface based on the mac provided"""
    for nic in nics:
        for nic_name, nic_mac in iter(nic.items()):
            if nic_mac == mac:
                return nic_name
    return None


def set_virtual_interface(port_type, nic, vlan, addr, base_vlan=None):
    """Create a vlan based virtual interface"""
    logger.info('Create a virtual interface for {} nic'.format(port_type))
    if port_type == 'pf' and base_vlan is not None:
        execute_shell_command('ip link add link {nic} name {nic}.{vlan} type '
                              'vlan id {vlan}'.format(nic=nic, vlan=base_vlan))
        logger.info('Bring the base virtual interface up')
        execute_shell_command('ip link set {nic}.{vlan} up'.format(
            nic=nic, vlan=base_vlan))
        nic = '{nic}.{vlan}'.format(nic=nic, vlan=base_vlan)

    execute_shell_command('ip link add link {nic} name {nic}.{vlan} type '
                          'vlan id {vlan}'.format(nic=nic, vlan=vlan))
    logger.info('Bring the virtual interface up')
    execute_shell_command('ip link set {nic}.{vlan} up'.format(
        nic=nic, vlan=vlan))
    logger.info('Set address {} on interface {}.{}'.format(addr, nic, vlan))
    execute_shell_command('ip addr add {addr} dev {nic}.{vlan}'.format(
        addr=addr, nic=nic, vlan=vlan))


def remove_virtual_interface(port_type, nic, vlan, base_vlan=None):
    """Delete a vlan based virtual interface"""
    logger.info('Delete virtual interface for {} nic'.format(port_type))
    if port_type == 'pf' and base_vlan is not None:
        vlan = base_vlan
    execute_shell_command('ip link del {nic}.{vlan}'.format(nic=nic,
                                                            vlan=vlan))


def main():
    logger.info('Start post net bootstrap script')
    ifaces = check_existing_interfaces()
    nic = choose_requested_interface(ifaces, args.mac)
    if args.add_iface:
        logger.info('Set and configure virtual interfaces')
        set_virtual_interface(args.port_type, nic, args.vlan, args.addr,
                              args.base_vlan)
    elif args.del_iface:
        remove_virtual_interface(args.port_type, nic, args.vlan,
                                 args.base_vlan)
    logger.info('Post net bootstrap script completed')


if __name__ == '__main__':
    main()



# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

from nfv_tempest_plugin.tests.common import shell_utilities as shell_utils
from nfv_tempest_plugin.tests.scenario import base_test
from oslo_log import log as logging
from tempest import config

CONF = config.CONF
LOG = logging.getLogger('{} [-] nfv_plugin_test'.format(__name__))


class TestLacpScenarios(base_test.BaseTest):
    def __init__(self, *args, **kwargs):
        super(TestLacpScenarios, self).__init__(*args, **kwargs)
        self.instance = None
        self.image_ref = CONF.compute.image_ref
        self.flavor_ref = CONF.compute.flavor_ref
        self.public_network = CONF.network.public_network_id

    def setUp(self):
        """Set up a single tenant with an accessible server

        If multi-host is enabled, save created server uuids.
        """
        super(TestLacpScenarios, self).setUp()
        """ pre setup creations and checks read from config files """

    def test_deployment_lacp(self, test='deployment_lacp', hypervisor_ip=None):
        """Check that lacp bonding is properly configure

        Configuration options example:
         - name: deployment_lacp
           bonding_config:
             - bond_name: 'dpdkbond1'
               bond_mode: 'balance-tcp'
               lacp_status: 'negotiated'
               lacp_time: 'fast'
               lacp_fallback_ab: 'true'
        """
        LOG.info('Starting deployment_lacp test.')

        if hypervisor_ip is None:
            hypervisor_ip = self._get_hypervisor_ip_from_undercloud(
                shell=CONF.nfv_plugin_options.undercloud_rc_file)[0]

        bonding_dict = {}
        test_setup_dict = self.test_setup_dict[test]
        if 'config_dict' in test_setup_dict and \
           'bonding_config' in test_setup_dict['config_dict']:
            bonding_dict = test_setup_dict['config_dict']['bonding_config'][0]

        cmd = 'sudo ovs-appctl bond/show {0} | '\
              'egrep "^bond_mode|^lacp_status|^lacp_fallback_ab"; '\
              'sudo ovs-appctl lacp/show {0} | '\
              'egrep "lacp_time"'.format(bonding_dict['bond_name'])
        output = shell_utils.run_command_over_ssh(hypervisor_ip, cmd) \
            .replace('\t', '').replace(' ', '').split('\n')
        bond_data = {}
        for i in range(len(output)):
            data = output[i].split(':')
            if len(data) == 2:
                bond_data[data[0]] = data[1]

        result = []
        checks = {'bond_mode', 'lacp_status', 'lacp_time', 'lacp_fallback_ab'}
        diff_checks_cmd = checks - set(bond_data.keys())
        diff_checks_cfg = checks - set(bonding_dict.keys())
        if len(diff_checks_cmd) > 0:
            result.append("Missing checks: {}. Check ovs commands "
                          "output".format(', '.join(diff_checks_cmd)))

        if len(diff_checks_cmd) > 0:
            result.append("Missing checks: {}. Check testcase config "
                          "file".format(', '.join(diff_checks_cfg)))

        for check in checks:
            if check not in diff_checks_cmd and \
               check not in diff_checks_cfg:
                if bond_data[check] != bonding_dict[check]:
                    result.append("Check failed: {}, Expected: {} - "
                                  "Found: {}".format(check,
                                                     bonding_dict[check],
                                                     bond_data[check]))
        self.assertTrue(len(result) == 0, '. '.join(result))
        return True

    def test_balance_tcp(self, test='balance_tcp'):
        """Test balance-tcp traffic distribution

        The method boots two instances connected through a balance_tcp bond,
        runs traffic between them and checks that traffic goes through the
        right interface
        * 1 flow: all the traffic through the same interface,
          the other one is not used
        * 2 flows: 50% of the traffic in each interface
        * 3 flows: 66% in one interface, 33% in the other one
        Configuration options example:
         - name: balance_tcp
           flavor: m1.medium.huge_pages_cpu_pinning_numa_node-0
           router: true
           package-names:
              - iperf
           bonding_config:
             - bond_name: 'dpdkbond1'
               ports: [ 'dpdk2', 'dpdk3']
        """
        LOG.info('Starting balance_tcp test.')

        if self.external_resources_data is None:
            raise ValueError('External resource data is required for the test')

        servers, key_pair = self.create_and_verify_resources(test=test)
        if len(servers) != 2:
            raise ValueError('The test requires 2 instances.')

        servers[0]['role'] = 'traffic_runner'
        servers[1]['role'] = 'listener'

        tests = [{'desc': '1 flow', 'iperf_option': '-P 1',
                  'threshold_1': 0, 'threshold_2': 2},
                 {'desc': '2 flows', 'iperf_option': '-P 2',
                  'threshold_1': 99, 'threshold_2': 101},
                 {'desc': '3 flows', 'iperf_option': '-P 3',
                  'threshold_1': 49, 'threshold_2': 51}]

        bonding_dict = {}
        test_setup_dict = self.test_setup_dict[test]
        if 'config_dict' in test_setup_dict and \
           'bonding_config' in test_setup_dict['config_dict']:
            bonding_dict = test_setup_dict['config_dict']['bonding_config'][0]

        for test in tests:
            receive_cmd = '(if pgrep iperf; then sudo pkill iperf; fi;' \
                          ' sudo iperf -s -u) > /dev/null 2>&1 &'
            srv = self.os_admin.servers_client.list_addresses(servers[1]['id'])
            server_addr = list(srv['addresses'].items())[1][1][0]['addr']
            send_cmd = '(if pgrep iperf; then sudo pkill iperf; fi;' \
                       ' sudo iperf -c {} {} -u -t 1000) > /dev/null 2>&1 &' \
                       .format(server_addr, test['iperf_option'])
            for srv in servers:
                cmd = send_cmd if 'traffic_runner' in srv['role'] \
                    else receive_cmd
                LOG.info('Executing iperf on {} - {}: {}'
                         .format(srv['role'], srv['fip'], cmd))
                ssh_source = self.get_remote_client(
                    srv['fip'], username=self.instance_user,
                    private_key=key_pair['private_key'])
                ssh_source.exec_command(cmd)

            # it may take some time to balance the traffic properly, so I give
            # 10 tries  to stabilize, usually is stabilized between try 1 and 2
            for i in range(1, 10):
                stats_begin = self.get_ovs_interface_statistics(
                    bonding_dict['ports'],
                    hypervisor=servers[0]['hypervisor_ip'])
                time.sleep(10)  # measured time
                stats_end = self.get_ovs_interface_statistics(
                    bonding_dict['ports'], stats_begin,
                    servers[0]['hypervisor_ip'])
                tx_pks_1 = stats_end[bonding_dict['ports'][0]]['tx_packets']
                tx_pks_2 = stats_end[bonding_dict['ports'][1]]['tx_packets']
                tx_pkts_max = max(tx_pks_1, tx_pks_2)
                tx_pkts_min = min(tx_pks_1, tx_pks_2)
                tx_pks_rel = 100 * tx_pkts_min / tx_pkts_max
                LOG.info('test: {}, try: {}, pks_1: {}, pks_2: {}, '
                         'tx_packets_rel: {}, threshold_1: {}, '
                         'threshold_2: {}'.format(test['desc'], i, tx_pks_1,
                                                  tx_pks_2, tx_pks_rel,
                                                  test['threshold_1'],
                                                  test['threshold_2']))
                if test['threshold_2'] >= tx_pks_rel >= test['threshold_1']:
                    break

            msg = "Traffic not well balanced. Value {} not between the " \
                  "thresholds: {} and {}".format(tx_pks_rel,
                                                 test['threshold_1'],
                                                 test['threshold_2'])
            result = test['threshold_2'] >= tx_pks_rel >= test['threshold_1']
            self.assertTrue(result, msg)

    def test_restart_ovs(self, test='restart_ovs'):
        """Test restart_ovs

        Check that config is loaded properly
        """
        LOG.info('Starting restart_ovs test.')

        hypervisor_ip = self._get_hypervisor_ip_from_undercloud(
            shell=CONF.nfv_plugin_options.undercloud_rc_file)[0]

        cmd = 'sudo systemctl restart openvswitch.service'
        shell_utils.run_command_over_ssh(hypervisor_ip, cmd)
        self.test_deployment_lacp(hypervisor_ip=hypervisor_ip)
